YUI.add('carousel', function (Y, NAME) {

"use strict";
/*

This is a generic carousel that's designed to handle proportional resizing of the viewport.
Here are the currently supported features:

* Moving 1 or more slides at a time
* Continuous motion through cloning of first and last slide sets
* Resize handling including optional proportional font re-sizing.
* Horizontal movement
* Configurable Auto advance
* Pagination and control generation

TODO Future additions:

* Vertical movement
* Data source plugins (Rather than expecting *all* markup up front)
* Handle irregular total of items.
* Keyboard navigation with arrow keys.

By defatult the slide width is determined by calculating the offsetWidth of the srcNode

Typical markup would be as follows:


<div id="carousel">
    <ul class="carousel">
        <li>Slide 1</li>
        <li>Slide 2</li>
        <li>Slide 3</li>
    </ul>
</div>

So in this case the srcNode would be #carousel. Should you need to override
the heights rather than use calculations then you can do so using the config parameter
"containerWidth"


*/

    /* Any frequently used shortcuts, strings and constants */
    var Node = Y.Node,
        NodeList = Y.NodeList;

    /* constructor */
    function Carousel() {
        Carousel.superclass.constructor.apply(this, arguments);
    }

    Carousel.NAME = "carousel";
    Carousel.ATTRS = {
        // Whether to auto advance slides or not
        autoPlay: { value: true },
        // Whether to preserve the aspect ration onResize.
        preserveAspectRatio: { value: false },
        // Describe the aspect ratio for fluid width
        // need to maintain an aspect ratio based on content
        aspectRatio: {
            value: null,
            setter: function(aspectRatio) {
                var arPieces,
                    logError;
                // Create error function
                logError = function() {
                };
                if (typeof aspectRatio === "object" && Y.Object.hasKey(aspectRatio, "width") && Y.Object.hasKey(aspectRatio, "height")) {
                    return aspectRatio;
                }
                // Parse config value in to object.
                if (aspectRatio && aspectRatio.indexOf(":") > -1){
                    arPieces = aspectRatio.split(":");
                    if (arPieces.length !== 2) {
                        logError();
                        return false;
                    }
                    return { "width": parseFloat(arPieces[0], 10), "height": parseFloat(arPieces[1], 10) };
                } else {
                    logError();
                }
                return false;
            }
        },
        noMqAspectRatio: { value: null },
        noMqNumSlidesVisible: { value: null },
        noMqNumSlidesAdvance: { value: null },
        // Selector to identify the carousel parent (usually a ul/ol).
        carouselClassName: { value: "carousel"},
        // Should the carousel continue automatically.
        isContinuous: { value: true },
        // Should fonts be resizeed relatively onResize?
        resizeFonts: { value: false },
        // Node selector to find nodes whose fonts
        // should be resized.
        fontNodes: {
            setter: function(sel) {
                var nl = Y.all(sel);
                return nl;
            }
        },

        // ContainerWidth. Needed for fontResizing.
        containerWidth: { value: null },
        // How many slides are visble at once
        numSlidesVisible: { value: 1 },
        // How many slides should we advance by in one go.
        numSlidesAdvance: { value: 1 },
        // numSlidesAdvance setting as it was at intialisation.
        defaultNumSlidesVisible: { value: null },
        // How many slides should we advance by in one go.
        defaultNumSlidesAdvance: { value: null },

        // How long should the animation last.
        slideAnimDuration: { value: 1 },
        // What interval should be used for advance the slides.
        slideAnimInterval: { value: 5000 },
        // What kind of easing should be used.
        slideAnimEasing: { value: Y.Easing.easeBoth },
        // Pixel Rounding - in some situations can help to unify rounding.
        // e.g: Math.floor or Math.ceil
        pixelRoundingFunc: { value: null },
        /*
        Allows for conf changes based on viewport size
        Checked at resize. Example:

        viewportConfig: {
            480: {
                numSlidesVisible: 3,
                numSlidesAdvance: 3,
                aspectRatio: "3:1",
            },
            720: {
                numSlidesVisible: 4,
                numSlidesAdvance: 4,
                aspectRatio: "4:1",
            },
            960: {
                numSlidesAdvance: 5,
                numSlidesVisible: 5,
                aspectRatio: "5:1",
            },
        }
        */
        viewportConfig: { value: null },
        // Ref node fo append/prepend/insertBefore/After pagination
        pagingAppendRefNode: {
            setter: function(sel) {
                var n = Y.one(sel);
                return n;
            }
        },
        // Where param should be one of before/after/prepend/append
        pagingAppendRefWhere: { value: null },
        // Max pagination items
        // When pagination exceeds this value it shouldn't be generated
        maxPagingItems: { value: 5 },
        // Ref node fo append/prepend/insertBefore/After nav
        controlsRefNode: {
            setter: function(sel) {
                var n = Y.one(sel);
                return n;
            }
        },
        // Where param should be one of before/after/prepend/append
        controlsRefWhere: { value: null },
        // Loading class name (removed after initialisation)
        // Classname is not added programmatically so needs to
        // be present in the markup.
        loadingClassname: { value: "loading" },
        // Loaded class name (added to the srcNode after initialisation)
        loadedClassname: { value: "loaded" },
        // Debug mode
        debug: { value: false }
    };


    /* Carousel extends the base Widget class */
    Y.extend(Carousel, Y.Widget, {

        initializer: function() {
            var defaultVpConf = {},
                validVpConfOverrides,
                noMqAspectRatio,
                noMqNumSlidesAdvance,
                noMqNumSlidesVisible,
                viewportConfig = this.get("viewportConfig"),
                preserveAspectRatio = this.get("preserveAspectRatio"),
                i, j, key;

            // Aspect ratio for old versions of IE that don't grok MQs
            if (preserveAspectRatio && viewportConfig && (Y.UA.ie && Y.UA.ie < 9)) {
                noMqAspectRatio = this.get("noMqAspectRatio");
                noMqNumSlidesVisible = this.get("noMqNumSlidesVisible");
                noMqNumSlidesAdvance = this.get("noMqNumSlidesAdvance");

                if (noMqAspectRatio && noMqNumSlidesVisible && noMqNumSlidesAdvance) {
                    this.set("aspectRatio", noMqAspectRatio);
                    this.set("numSlidesAdvance", noMqNumSlidesAdvance);
                    this.set("numSlidesVisible", noMqNumSlidesVisible);
                } else {
                    throw new Error("please configure explicit values for noMqAspectRatio/noMqNumSlidesAdvance/noMqNumSlidesVisible");
                }
            }

            this.set("defaultNumSlidesAdvance", this.get("numSlidesAdvance"));
            this.set("defaultNumSlidesVisible", this.get("numSlidesVisible"));


            this.set("defaultAspectRatio", this.get("aspectRatio"));
            validVpConfOverrides = this.validVpConfOverrides = ["numSlidesVisible", "numSlidesAdvance", "aspectRatio", "autoPlay"];
            for (i=0, j=validVpConfOverrides.length; i < j; i++) {
                key = validVpConfOverrides[i];
                defaultVpConf[key] = this.get(key);
            }
            this.defaultVpConf = defaultVpConf;
        },

        destructor : function() {
            /*
             * destructor is part of the lifecycle introduced by
             * the Widget class. It is invoked during destruction,
             * and can be used to cleanup instance specific state.
             *
             * Anything under the boundingBox will be cleaned up by the Widget base class
             * We only need to clean up nodes/events attached outside of the bounding Box
             *
             * It does not need to invoke the superclass destructor.
             * destroy() will call initializer() for all classes in the hierarchy.
             */
        },

        renderUI : function() {
            /*
             * renderUI is intended to be used by the Widget subclass
             * to create or insert new elements into the DOM.
             */
            var nodeListSlides,
                srcNode = this.get("srcNode"),
                carouselClassName = this.get("carouselClassName"),
                isContinuous = this.get("isContinuous"),
                loadingClass = this.get("loadingClassname"),
                loadedClass = this.get("loadedClassname");


            this.IE = Y.UA.ie;
            this.interval = null;
            this.curSlide = 0;

            nodeListSlides = this.getCarouselNodeList();
            this.nodeCarousel = srcNode.one("."+ carouselClassName);
            this.numSlides = nodeListSlides.size();

            this.caroAnim = new Y.Anim({
                node: this.nodeCarousel
            });
            this.caroAnim.set("duration", this.get("slideAnimDuration"));
            this.caroAnim.set("easing", this.get("slideAnimEasing"));
            this.caroAnim.on("end", function(){
                this.flip();
                this.updatePagination();
            }, this);

            // Always run the resize calcs at initialisation
            this.handleResize();
            if (this.get("autoPlay")) {
                this.autoPlay();
            }

            // Create next/prev
            this.generateControls();
            // Create Paging
            this.generatePagination();


            // If we are wanting a continuous carousel
            if (isContinuous === true) {
                this.cloneSlides();
            }

            // We are ready remove the loading classname.
            this.get("boundingBox").all("." + loadingClass).removeClass(loadingClass);
            srcNode.addClass(loadedClass);

        },

        bindUI : function() {
            Y.on('windowresize', Y.bind(this.handleResize, this));
        },

        syncUI : function() {
            var isContinuous = this.get("isContinuous");
            this.updatePagination();
            if (isContinuous === true) {
                this.gotoSlide(0, 0);
            }
        },

        // Beyond this point is the Carousel specific application and rendering logic

        getCarouselNodeList: function() {
            var srcNode = this.get("srcNode"),
                carouselClassName = this.get("carouselClassName");
            return srcNode.all("." + carouselClassName + " > li");
        },

        updatePagination: function() {
            var srcNode = this.get("srcNode"),
                numSlidesVisible = this.get("numSlidesVisible"),
                paginationRef = this.get("pagingAppendRefNode") || srcNode,
                paginationListItems = paginationRef.get("parentNode").all(".pagination li a");

            paginationListItems.each(function(a){
                var nodeClassName = a.get("className");
                if (parseInt(nodeClassName.replace("p-", ""), 10) === Math.ceil(parseInt(this.curSlide / numSlidesVisible, 10))) {
                    a.addClass("active");
                } else {
                    a.removeClass("active");
                }
            }, this);

        },

        /*
         * insert a node based on ref and where
         * Defaults to appending to ref.
         *
         */
        insertItem: function(node, ref, where) {
            switch (where) {
                case "prepend":
                    ref.prepend(node);
                    break;
                case "before":
                    ref.insert(node, "before");
                    break;
                case "after":
                    ref.insert(node, "after");
                    break;
                default:
                    ref.append(node);
            }
        },

        generatePagination: function() {
            var pageText = 'Page',
                li, a, sp, txt, olNode, ol,
                numSlidesVisible = this.get("numSlidesVisible"),
                pagingAppendRefNode = this.get("pagingAppendRefNode") || this.get("srcNode"),
                pagingAppendRefWhere = this.get("pagingAppendRefWhere"),
                maxPagingItems = this.get("maxPagingItems"),
                numPages,
                existingPagination, i;

            // Kill pagination if it already exists.
            existingPagination = pagingAppendRefNode.get("parentNode").one(".pagination");
            if (existingPagination) {
                existingPagination.remove();
            }
            numPages = Math.ceil(this.numSlides / numSlidesVisible);
            // Generate it if it's under the max paging items
            if (numPages <= maxPagingItems) {
                ol = document.createElement('ol');
                ol.className = "pagination";
                for (i=0; i < numPages; i++){
                    li = document.createElement('li');
                    a = document.createElement('a');
                    sp = document.createElement('span');
                    a.href = '#';
                    txt = document.createTextNode(pageText+(i+1));
                    sp.appendChild(txt);
                    a.appendChild(sp);
                    a.className = 'p-'+i;
                    li.appendChild(a);
                    ol.appendChild(li);
                }

                this.insertItem(ol, pagingAppendRefNode, pagingAppendRefWhere);

                olNode = Y.one(ol);
                // delegate the clicks on the olNode.
                olNode.delegate("click", this.handlePageClick, 'a', this);
            }
        },

        handlePageClick: function(e) {
            var targetClass,
                numSlidesAdvance = this.get("numSlidesAdvance");
            e.preventDefault();
            if (this.caroAnim.get("running") === false) {
                if (e && this.autoPlayTimer) {
                    window.clearInterval(this.autoPlayTimer);
                }
                targetClass = e.currentTarget.get("className");
                this.gotoSlide(parseInt(targetClass.replace("p-", ""), 10) * numSlidesAdvance);
            }
        },

        generateControls: function() {
            var prev = Node.create('<a href="#" class="prev"><span>Previous Slide</span></a>'),
                next = Node.create('<a href="#" class="next"><span>Next Slide</span></a>'),
                controlsRefNode = this.get("controlsRefNode") || this.get("srcNode"),
                controlsRefWhere = this.get("controlsRefWhere");

            next.on("click", this.next, this);
            prev.on("click", this.prev, this);

            this.insertItem(prev, controlsRefNode, controlsRefWhere);
            this.insertItem(next, controlsRefNode, controlsRefWhere);
        },

        advance: function(e, val){
            if (e) {
                e.preventDefault();
            }
            if (e && this.autoPlayTimer) {
                window.clearInterval(this.autoPlayTimer);
            }
            if (this.caroAnim.get("running") === true) {
                return;
            }
            this.gotoSlide(val);
        },

        /*
         * Show the next slide
         */
        next: function(e){
            this.advance(e, this.curSlide + this.get("numSlidesAdvance"));
        },

        /*
         * Show the previous slide
         */
        prev: function(e){
            this.advance(e, this.curSlide - this.get("numSlidesAdvance"));
        },

        /*
         * Goto slide n (zero-indexed)
         */
        gotoSlide: function(nSlideIndex, duration) {
            var xOffSet,
                cloneSlide,
                isContinuous = this.get("isContinuous"),
                numSlidesAdvance = this.get("numSlidesAdvance"),
                slideWidth = this.slideWidth;

            this.requestedIndex = nSlideIndex;

            duration = (duration === 0) ? duration : (duration || this.get("slideAnimDuration"));
            if (nSlideIndex < 0) {
                if (isContinuous === true) {
                    // Fold over to the start of the cloned slides
                    this.animateTo(0, duration);
                    this.curSlide = this.numSlides - numSlidesAdvance;
                } else {
                    // Fast-forward to the previous end.
                    this.animateTo(-(slideWidth * (this.numSlides - numSlidesAdvance)), duration);
                    this.curSlide = this.numSlides - numSlidesAdvance;
                }
            } else if (nSlideIndex >= (this.numSlides)) {
                if (isContinuous === true) {
                    xOffSet = -((slideWidth) * (this.numSlides + numSlidesAdvance));
                    this.animateTo(xOffSet, duration);
                    this.curSlide = this.numSlides;
                } else {
                    this.animateTo(0, duration);
                    this.curSlide = 0;
                }
            } else {
                cloneSlide = (isContinuous === true) ? numSlidesAdvance : 0;
                xOffSet = -(slideWidth * (nSlideIndex + cloneSlide));
                this.animateTo(xOffSet, duration);
                this.curSlide = nSlideIndex;
            }

        },

        autoPlay: function() {
            var that = this;
            this.autoPlayTimer = window.setInterval(function(){
               that.next();
            }, that.get("slideAnimInterval"));
        },

        animateTo: function(value, duration) {
            var origDuration;
            if (duration === 0) {
                this.nodeCarousel.setStyle("left", value);
            } else {
                if (duration) {
                    origDuration = this.caroAnim.get("duration");
                    this.caroAnim.set("duration", duration);
                }
                this.caroAnim.set("to", { "left": value });
                this.caroAnim.run();
                this.caroAnim.set("duration", origDuration);
            }
        },

        transitionTo: function(value, duration) {
            this.nodeCarousel.transition({
                duration: duration,
                left: value + "px"
            });
        },

        flip: function() {
            var isContinuous = this.get("isContinuous"),
                numSlidesAdvance = this.get("numSlidesAdvance"),
                slideWidth,
                xOffSet;
            if (isContinuous === true && this.requestedIndex) {
                slideWidth = this.slideWidth;
                if (this.requestedIndex < 0) {
                    xOffSet = -(slideWidth * (this.numSlides));
                    this.animateTo(xOffSet, 0);
                    this.curSlide = this.numSlides - numSlidesAdvance;
                } else if (this.requestedIndex > (this.numSlides - 1)) {
                    this.animateTo(-(slideWidth * numSlidesAdvance), 0);
                    this.curSlide = 0;
                }
            }
        },

        /*
         * Clones the first and last slides so we can display the
         * slideshow continuously.
         */
        cloneSlides: function(){
            var clones,
                firstSlide,
                lastSlide,
                firstCloneList,
                lastCloneList,
                carouselNodeList,
                numSlidesVisible = this.get("numSlidesVisible"),
                carouselNodeListSize,
                srcNode = this.get("srcNode"), i, j, k, l;

            clones = srcNode.all(".clone");

            if (clones.size() === 0 || (clones.size() / 2) !== numSlidesVisible) {
                clones.remove(true);
                carouselNodeList = this.getCarouselNodeList();
                carouselNodeListSize = carouselNodeList.size();

                // Get references to first and last slides.
                firstSlide = carouselNodeList.item(0);
                lastSlide = carouselNodeList.item(this.numSlides-1);

                // Create new nodelists to store clones.
                firstCloneList = new NodeList([]);
                lastCloneList = new NodeList([]);

                // Grab clones of last n slides
                for (i=carouselNodeListSize - numSlidesVisible, j=carouselNodeListSize; i<j; i++) {
                    lastCloneList.push(carouselNodeList.item(i).cloneNode(true));
                }
                // Grab clones of first n slides
                for (k=0, l=numSlidesVisible; k<l; k++) {
                    firstCloneList.push(carouselNodeList.item(k).cloneNode(true));
                }

                // Add class as a way to identify the clones.
                firstCloneList.addClass("clone");
                lastCloneList.addClass("clone");

                // Add the cloned slides to either end of the slide list
                lastSlide.insert(firstCloneList, "after");
                firstSlide.insert(lastCloneList, "before");

            }
        },

        /*
         * Applies configoverrides assuming config properties are
         * within the validVpConfOverrides list.
         */
        applyConfigOverrides: function(confObj) {
            var confKey;
            Y.Object.each(confObj, function(v,k){
                confKey = k;
                // Check item is in the list of overridable keys.
                if (Y.Array.indexOf(this.validVpConfOverrides, confKey) > -1 ) {
                    this.set(k,v);
                } else {
                    throw new Error("viewportConfKey: "+confKey +" is not a valid key");
                }
            }, this);
        },


        /*
         * Handle resize events gracefully so we can
         * scale the carousel accordingly.
         */
        handleResize: function(){
            var percDiffWidth,
                relativeHeight,
                srcNode = this.get("srcNode"),
                carouselNodeList = this.getCarouselNodeList(),
                curContainerWidth,
                aspectRatio,
                numSlidesAdvance,
                numSlidesVisible,
                pixelRoundingFunc,
                newHeight,
                newWidth,
                fontNodes,
                viewportConfig = this.get("viewportConfig"),
                overridesObject,
                vpWidth,
                widthKeys,
                containerWidth = this.get("containerWidth"),
                preserveAspectRatio = this.get("preserveAspectRatio"),
                resizeFonts = this.get("resizeFonts"),
                slideToMoveTo,
                curSlideModulo,
                i, j;

            if (preserveAspectRatio) {

                // Ignore viewPort config if IE < 9
                if (viewportConfig && (!Y.UA.ie || Y.UA.ie >= 9)) {
                    /* Handle config overrides as supplied in config.
                    * Loops through conf in reverse to match largest
                    * width first as these confs match given
                    * resolution or higher.
                    */
                    vpWidth = Y.one("body").get("winWidth");
                    widthKeys = Y.Object.keys(viewportConfig);
                    widthKeys.sort(Y.Array.numericSort);
                    // These are the defaults
                    overridesObject = this.defaultVpConf;
                    for (i=0, j=widthKeys.length; i < j; i++ ) {
                        if (vpWidth >= widthKeys[i]) {
                            // for each conf that matches extend the conf object
                            // to provide cascading inheritance.
                            // TODO: Might consider caching each step of
                            // this if performance is an issue.
                            // though should be fairly quick as there's a minimal
                            // set of keys that can be overriden.
                            overridesObject = Y.merge(overridesObject, viewportConfig[widthKeys[i]]);
                        }

                    }
                    // Apply the configurations changes.
                    this.applyConfigOverrides(overridesObject);
                }

                // Check these values now in case view port config changed the values.
                aspectRatio = this.get("aspectRatio");
                numSlidesAdvance = this.get("numSlidesAdvance");
                numSlidesVisible = this.get("numSlidesVisible");

                // Container measurements now
                curContainerWidth = srcNode.get("offsetWidth");

                // Get height relative to width change
                relativeHeight = parseInt((curContainerWidth / aspectRatio.width) * aspectRatio.height, 10);
                srcNode.setStyle("height", relativeHeight+"px");

                newHeight = relativeHeight;
                newWidth = (curContainerWidth / numSlidesVisible);

                pixelRoundingFunc = this.get("pixelRoundingFunc");
                if (pixelRoundingFunc) {
                    newHeight = pixelRoundingFunc(newHeight);
                    newWidth = pixelRoundingFunc(newWidth);
                }
                carouselNodeList.each(function(slideNode){
                    slideNode.setStyle("width", newWidth);
                    slideNode.setStyle("height", newHeight);
                });

                this.slideWidth = newWidth;
                this.slideHeight = newHeight;

                // We could do this with an event but want to ensure linear handling of
                // cloning within the rest of this flow.
                // It'll only do work if the number of clones/2 is != numSlidesVisible.
                if (this.get("isContinuous")) {
                    this.cloneSlides();
                }
                this.generatePagination();

                // If resize fonts is turned on
                if (resizeFonts === true) {
                    percDiffWidth = Math.round((((100 / containerWidth) * curContainerWidth)*100)) / 100;
                    // Updates the font-size based on the current width and the original
                    // font-size when we started.
                    fontNodes = this.get("fontNodes");
                    fontNodes.each(function(node){
                        var origFontSize = this.getFontSize(node);
                        if ( percDiffWidth === 100 ) {
                            node.setStyle("fontSize", origFontSize + "px");
                        } else {
                            node.setStyle("fontSize", (parseInt(origFontSize, 10) / 100) * percDiffWidth + "px");
                        }
                    }, this);
                }


                // Put us back to where we should be.
                slideToMoveTo = this.curSlide;
                curSlideModulo = (slideToMoveTo % numSlidesVisible);
                if (curSlideModulo !== 0) {
                    slideToMoveTo = slideToMoveTo - curSlideModulo;
                }
                if (this.caroAnim.get("running") === false) {
                    this.gotoSlide(slideToMoveTo, 0);
                } else {
                    // Single event so it's detached immediately when fired.
                    this.caroAnim.once("end", function(){
                        this.gotoSlide(slideToMoveTo, 0);
                    }, this);
                }

            }

        },

        /*
         * Get's the font-size of a node and stashes it's value as a custom
         * attr on the first run. Subsequent lookups return the attr value.
         */
        getFontSize: function(node){
            var origFontSize = node.getAttribute("origFontSize"),
                curStyle;
            if (!origFontSize) {
                if (this.IE && this.IE < 9) {
                    // Fix to actually get a font-size from ie < 9 that reflects reality
                    curStyle = node.get("currentStyle");
                    origFontSize = (parseInt(curStyle.fontSize, 10)/100 * 13);
                } else {
                    origFontSize = parseInt(node.getStyle("fontSize"), 10);
                }
                node.setAttribute("origFontSize", origFontSize);
            }
            return origFontSize;
        }

    });

    Y.namespace('u1');
    Y.u1.Carousel = Carousel;


}, '@VERSION@', {
    "requires": [
        "widget-base",
        "widget-htmlparser",
        "widget-uievents",
        "widget-skin",
        "anim-base",
        "anim-color",
        "anim-curve",
        "anim-easing",
        "anim-node-plugin",
        "anim-scroll",
        "anim-xy",
        "event-resize",
        "event-touch",
        "transition",
        "substitute"
    ]
});
