YUI.add('files-uploader', function (Y, NAME) {

 "use strict";

var FilesUploader,

    BINDUI          = 'bindUI',
    RENDERUI        = 'renderUI';


/**
* Extends uploader-html5 with a frontend and functionality.
*
* @module one-uploader
*/
FilesUploader = Y.Base.create('oneFilesUploader', Y.U1.Widgets.Uploader, [Y.U1.Widgets.UploaderFollow], {

    initializer: function () {

        Y.before(this.renderFilesUI, this, RENDERUI);
        Y.before(this.bindFilesUI, this, BINDUI);

        this.after('render', function (){

            // move the selectFilesButton
            // nuke the default style
            // TODO: override template for this instance?
            this.get('contentBox').one('.uploader-controls').prepend(this.get('selectFilesButton').remove().setAttribute('style', ''));

        }, this);

    },

    destructor: function () {

        if(this.get('simpleUploader')) {
            this.get('simpleUploader').show();
        }

    },

    /**
    * Renders the ui
    *
    * @method renderFilesUI
    */
    renderFilesUI: function () {

        // dom structure
        this.renderBase();

        this._filesDropAreaStatusChange();
        this._filesStatusChange();

        if(this.get('simpleUploader')) {
            this.get('simpleUploader').hide();
        }

    },

    /**
    * Appends a skeleton to the contentBox
    * Will also throw Error if something important is missing
    *
    * @method renderBase
    */
    renderBase: function () {

        var cb = this.get('contentBox');

        cb.append(FilesUploader.TEMPLATES.base({
            noFilesSelectedLabel:   this.get('strings.noFilesSelected'),
            cancelLabel:            this.get('strings.cancel'),
            uploadLabel:            this.get('strings.upload')
        }));

    },

    /**
    * Renders a file inside it's row
    * Uses `TEMPLATES.file`
    *
    * @param file {Y.File}
    * @return HTML
    * @method renderFile
    * @public
    */
    renderFile: function (file) {

        var percent = Math.min(100, Math.round(10000*file.get('bytesUploaded')/file.get('size'))/100);

        if(!file.get('node')) {
            return;
        }

        if(isNaN(percent)) {
            percent = 0;
        }

        file.get('node').setHTML(FilesUploader.TEMPLATES.file({
            host: this,
            file: file,
            percent: percent
        }));

        this.fire('filerender', { file: file });
    },

    createFileNode: function (file) {

        var node = Y.Node.create(FilesUploader.TEMPLATES.fileRow({
            fileId: file.get('id')
        }));

        if(typeof file.get('node') === 'undefined') {
            file.addAttr('node', {
                value: node
            });
        }

    },

    clearFilesDOM: function () {

        Y.each(this.get('fileList'), function (file) {

            file.set('node', null);

        }, this);

        this.get('contentBox').one('.uploader-files').empty();

    },

    /**
    * @method bindFilesUI
    */
    bindFilesUI: function () {

        var cb = this.get('contentBox');

        cb.delegate('click', this.cancel,                   '.uploader-cancel', this);
        cb.delegate('click', this._complete,                '.uploader-dismiss', this);
        cb.delegate('click', this.start,                    '.uploader-start', this);

        cb.delegate('click', this._filesCancelFileClick,    '.file-cancel', this);
        cb.delegate('click', this._filesRetryClick,         '.file-retry', this);

        this.after('filelistrender',            this.alignHeader);
        this.after('fileselect',                this._filesSelectFiles);

        this.on('enabledChange',                this._filesEnabledChange);
        this.on('fileListChange',               this._filesFileListChange);
        this.after('statusChange',              this._filesStatusChange);
        this.after('dropAreaStatusChange',      this._filesDropAreaStatusChange);

    },

    /**
    * Handler for file retries
    *
    * @method _filesRetryClick
    * @private
    */
    _filesRetryClick: function (e) {

        var id = e.target.ancestor('li').getData('id');

        this.retryFile(this.getFile(id));

    },

    /**
    *
    * @method _filesEnabledChange
    * @private
    */
    _filesEnabledChange: function (e) {

        this.get('contentBox').one('.uploader-start').set('disabled', !e.newVal);
        this.get('selectFilesButton').set('disabled', !e.newVal);

    },

    /**
    *
    * @method _filesFileListChange
    * @private
    */
    _filesFileListChange: function (e) {

        var cb = this.get('contentBox'),
            files = cb.one('.uploader-files'),
            noFiles = cb.one('.uploader-nofiles');

        if(e.newVal.length === 0) {
            files.hide();
            noFiles.show();
            this.clearFilesDOM();
        } else {
            files.show();
            noFiles.hide();
        }

        // if pending, allow user to upload
        if(!this.haveProgressingFiles()) {
            cb.one('.uploader-start').set('disabled', e.newVal.length === 0);
        }

    },

    /**
    *
    * Updates status node whenever the status was changed
    *
    * @method _filesStatusChange
    * @private
    */
    _filesStatusChange: function () {

        this.get('contentBox').one('.uploader-status').setHTML(this.get('status.text'));

    },

    _filesDropAreaStatusChange: function () {

        this.get('contentBox').one('.uploader-dd-area').set('text', this.get('dropAreaStatus'));

    },


    /**
    * @method alignHeader
    * @private
    */
    alignHeader: function () {

        // make header columns match up with file columns
        var headerWidth     = this.get('contentBox').one('.uploader-header').get('clientWidth'),
            fileRow         = this.get('contentBox').one('.uploader-files'),
            fileRowWidth    = fileRow ? fileRow.get('clientWidth') : 0;

        if(fileRowWidth > 0) {
            this.get('contentBox').one('.uploader-header').setStyle('paddingRight', (headerWidth-fileRowWidth) +'px');
        }

    },

    /**
    *
    *
    * @oaram event {Event}
    * @method _filesSelectFiles
    * @private
    */
    _filesSelectFiles: function (event) {

        var files       = event.fileList,
            filesNode   = this.get('contentBox').one('.uploader-files'),
            perFileVars = {};

        Y.each(files, function (file) {

            this.createFileNode(file);

            this.renderFile(file);

            filesNode.append(file.get('node'));

            file.after(['statusChange', 'bytesUploadedChange'], function (e) {
                this.renderFile(e.target);
            }, this);

            perFileVars[file.get('id')] = {
                parent_key: this.get('current_folderid'),
                base: this.get('current_path'),
                path: this.get('path'),
                csrfmiddlewaretoken: this.get('csrfmiddlewaretoken'),
                redirect: this.get('redirect')
            };
        }, this);

        this.set("postVarsPerFile", Y.merge(this.get("postVarsPerFile"), perFileVars));

        this.fire('filelistrender');

    },

    /**
    * Handler for when a file is cancelled
    * Removes file from fileList, and queue if active. If in a queue it is also cancelled
    * Manually updates the uploader's total uploaded bytes
    *
    * @method _filesCancelFileClick
    * @private
    */
    _filesCancelFileClick: function (e) {

        var fileRow     = e.target.ancestor('li'),
            file        = this.getFile(fileRow.getData('id'));

        this.cancelFile(file);

        this.renderFile(file);

        this.fire('filelistrender');

    }

}, {
    ATTRS: {
        csrfmiddlewaretoken: { value: '' },
        current_folderid: { value: '' },
        current_path: { value: '' },
        multipleFiles: { value: true },
        path: { value: '' },
        redirect: { value: 'False' },
        simpleUploader: {
            value: null,
            setter: function (val) {
                return Y.Lang.isString(val) ? Y.one(val) : val;
            }
        }
    },

    TEMPLATES: {

        base: Y.Template.Micro.revive(Y.U1.Templates['t-uploader-base']),
        file: Y.Template.Micro.revive(Y.U1.Templates['t-uploader-file']),
        fileRow: Y.Template.Micro.revive(Y.U1.Templates['t-uploader-filerow'])

    }
});

Y.namespace('U1.Widgets').FilesUploader = FilesUploader;


}, '@VERSION@', {"requires": ["uploader", "uploader-follow", "files-uploader-templates", "template", "transition"]});
