YUI.add('imageloader', function (Y, NAME) {

 "use strict";

    /**
     * The ImageLoader Utility is a framework to dynamically load images
     * according to certain triggers, enabling faster load times and a more
     * responsive UI.
     *
     * U1 imageloader module which is an augmented version of the YUI version
     * to allow the setting of a timer which dampens the scroll handler to only
     * be called every n ms and, if using this damping value, only loads images
     * within the viewport. Otherwise exactly the same as the YUI imageloader.
     *
     * In other words, if you don't set the damping value, U1 imageloader
     * behaves exactly like YUI imageloader.
     *
     * @module one-imageloader
     */


    /**
     *
     */
    var U1ImgLoadGroup = function() {},
        U1ImgLoadImgObj = function() {};

    U1ImgLoadGroup.prototype._didFoldCheck = false;
    U1ImgLoadGroup.prototype._hasRegisteredImg = false;

    /**
     * new attribute to hold the delay value for checking position relative to
     * fold
     */
    U1ImgLoadGroup.ATTRS = {
        foldCheckDelay: {
            value: null,
            setter: function(delay) {
                this._foldCheckDelay = delay; return delay;
            },
            lazyAdd: false
        }

    };

    /**
     * Registers an image with the group.
     * Arguments are passed through to a <code>Y.U1MS.ImgLoadImgObj</code> constructor; see that class' attribute documentation for detailed information. "<code>domId</code>" is a required attribute.
     * @method registerImage
     * @param {Object} *  A configuration object literal with attribute name/value pairs  (passed through to a <code>Y.U1MS.ImgLoadImgObj</code> constructor)
     * @return {Object}  <code>Y.U1MS.ImgLoadImgObj</code> that was registered
     */
    U1ImgLoadGroup.prototype.registerImage = function() {
        var domId = arguments[0].domId;
        if (! domId) {
            return null;
        }

        Y.log('adding image with id: ' + domId + ' to group: ' + this.get('name'), 'info', 'imageloader');

        this._imgObjs[domId] = new Y.U1.ImgLoadImgObj(arguments[0]);
        this._hasRegisteredImg = true;
        return this._imgObjs[domId];
    };


    U1ImgLoadGroup.prototype._foldCheck = function() {
        Y.log('Checking for images above the fold in group: "' + this.get('name') + '"', 'info', 'imageloader');

        var allFetched = true,
            viewReg = Y.DOM.viewportRegion(),
            hLimit = viewReg.bottom + this.get('foldDistance'),
            tLimit = viewReg.top - this.get('foldDistance'),
            id, imgFetched, els, i, len, vLimit;

        // Check something has been registered.
        if (this._hasRegisteredImg === false && !this._className) {
            Y.log('Nothing has been registered yet for this group. Bailing...');
            return;
        }

        // check if we already did a fold check since foldCheckDelay
        if (this._didFoldCheck) {
            Y.log('Already done a foldCheck, bailing out...');
            return;
        }

        // unless we've uncovered new frontiers, there's no need to continue
        if (hLimit <= this._maxKnownHLimit && !Y.Lang.isNumber(this._foldCheckDelay)) {
            Y.log('Within hLimit and no foldCheckDelay, exiting', 'info', 'imageloader:_foldcheck');
            return;
        }
        this._maxKnownHLimit = hLimit;

        vLimit = Y.Lang.isNumber(this._foldCheckDelay) ? [hLimit, tLimit] : hLimit;

        for (id in this._imgObjs) {
            if (this._imgObjs.hasOwnProperty(id)) {
                imgFetched = this._imgObjs[id].fetch(vLimit);
                allFetched = (allFetched && imgFetched);
            }
        }

        // and by class
        if (this._className) {
            if (this._classImageEls === null) {
                // get all the relevant elements and store them
                this._classImageEls = [];
                els = Y.all('.' + this._className);
                els.each( function(node) { this._classImageEls.push( { el: node, y: node.getY(), fetched: false } ); }, this);
            }
            els = this._classImageEls;
            for (i=0, len = els.length; i < len; i++) {
                if (els[i].fetched) {
                    continue;
                }
                if ((els[i].y && els[i].y <= hLimit) && (els[i].y && els[i].y >= tLimit)) {
                    //els[i].el.removeClass(this._className);
                    this._updateNodeClassName(els[i].el);
                    els[i].fetched = true;
                    Y.log('Image with id "' + els[i].el.get('id') + '" is within distance of the fold. Fetching image. (Image registered by class name with the group - may not have an id.)', 'info', 'imageloader');
                }
                else {
                    allFetched = false;
                }
            }
        }

        // if allFetched, remove listeners
        // checkedFlag ensures something was looked at preventing a fall-through error in IE8.
        // Which occurs if we've not actually checked anything.
        if (allFetched) {
            Y.log('All images fetched; removing listeners for group: "' + this.get('name') + '"', 'info', 'imageloader');
            this._clearTriggers();
        }

        // U1 FOLD DELAY

        /**
         * check if we've done a fold check in the last n ms, and if so, don't
         * check again
         */
        this._didFoldCheck = true;
        // prevent fetching for next 'foldCheckDelay' milliseconds
        this._foldCheckTimer = window.setTimeout(Y.bind(function() {
            // allow fold check
            this._didFoldCheck = false;
            this._foldCheck();
            // tidy up
            window.clearTimeout(this._foldCheckTimer);
            this._didFoldCheck = false;
        }, this), this._foldCheckDelay);
    };


    //------------------------------------------------


    /**
     * update fetch to work on a range rather than just a simple limit (the YUI
     * version grabs everything above the viewport bottom, we only want what's in
     * the viewport range)
     */



    U1ImgLoadImgObj.prototype.fetch = function(withinY) {
        if (this._fetched) {
            return true;
        }

        var el = this._getImgEl(),
        yPos;
        if (! el) {
            return false;
        }

        if (withinY) {
            // need a distance check
            yPos = this._getYPos();
            // U1 FOLD RANGE
            if (Y.Lang.isArray(withinY) && (!yPos || yPos > withinY[0] || yPos < withinY[1])) {
                return false;
            } else if (! yPos || yPos > withinY) {
                return false;
            }
            //Y.log('Image with id "' + this.get('domId') + '" is within distance of the fold. Fetching image.', 'info', 'imageloader');

        }

        //Y.log('Fetching image with id "' + this.get('domId') + '".', 'info', 'imageloader');

        // apply url
        if (this.get('bgUrl') !== null) {
            // bg url
            if (this.get('isPng') && Y.UA.ie && Y.UA.ie <= 6) {
                // png for which to apply AlphaImageLoader
                el.setStyle('filter', 'progid:DXImageTransform.Microsoft.AlphaImageLoader(src="' + this.get('bgUrl') + '", sizingMethod="' + this.get('sizingMethod') + '", enabled="' + this.get('enabled') + '")');
            }
            else {
                // regular bg image
                el.setStyle('backgroundImage', "url('" + this.get('bgUrl') + "')");
            }
        }
        else if (this.get('srcUrl') !== null) {
            // regular src image
            el.setAttribute('src', this.get('srcUrl'));
        }

        // apply attributes
        if (this.get('setVisible')) {
            el.setStyle('visibility', 'visible');
        }
        if (this.get('width')) {
            el.setAttribute('width', this.get('width'));
        }
        if (this.get('height')) {
            el.setAttribute('height', this.get('height'));
        }

        this._fetched = true;

        return true;

    };

    Y.namespace("U1");
    Y.U1.ImgLoadGroup = Y.Base.mix(Y.ImgLoadGroup, [U1ImgLoadGroup]);
    Y.U1.ImgLoadImgObj = Y.Base.mix(Y.ImgLoadImgObj, [U1ImgLoadImgObj]);


}, '@VERSION@', {
    "requires": [
        "base-base",
        "base-pluginhost",
        "base-build",
        "node-base",
        "node-event-delegate",
        "node-pluginhost",
        "node-screen",
        "node-style",
        "imageloader"
    ]
});
