YUI.add('photos-gallery-app', function (Y, NAME) {

"use strict";
var ONEERROR = 'one:error',
    ONEINFO = 'one:info',

    PhotosGalleryApp,

    // default slideshow interval time in ms
    SLIDESHOWINTERVAL = 3000;


/**
The app

@class PhotosGalleryApp
@constructor
@extends App
@beta

**/

PhotosGalleryApp = Y.Base.create('photosGalleryApp', Y.App, [Y.Photos.PhotosCommon], {
    views: {
        'gallery': {
            type: Y.Photos.GalleryView
        }
    },

    /**
    initializer runs once when the {{#crossLink "PhotosGalleryApp"}}{{/crossLink}} is constructed

    <ul>
        <li>Makes an Ajax request for photos</li>
        <li>Subscribes `onBusy`, `onIdle`, keyboard presses and emitter events to handlers</li>
        <li>Does manual dispatch if an url is not automatically dispatched (back button in browser)</li>
        <li>Creates a {{#crossLink "PhotoView"}}{{/crossLink}} and sets {{#crossLink "PhotosGalleryApp"}}{{/crossLink}} as bubble target</li>
    </ul>

    @method initializer
    @param {Object} [config]* on construction config
        @param {String} [config.nodekey] The current folder nodekey
        @param {String} [config.start] Photos start index
    **/

    initializer: function (config) {
        var photoList = this.get('photoList');

        Y.log('New GalleryApp', 'note', 'PhotosGalleryApp.initializer');

        // template generating URLS
        // changes whether the gallery is public or not
        this.photoURLTemplate = Y.one('#photo-url-template').getContent().replace(/\s+/, '');

        this.render().showContent('.photos-gallery-app', {
            view: {
                name: 'gallery',
                config: {
                    photoList: photoList
                }
            }
        });

        // subscribe to photoList events
        photoList.addTarget(this);

        this.after('photoList:current_photoChange', this.afterCurrentPhotoChange);
        this.after('photoList:reset', this.afterPhotosReset);

        // routes
        this.route(config.albumURLScheme, [
            'closeGallery'
        ]);
        this.route(config.galleryURLScheme, [
            'handleAlbum',
            'handlePhoto',
            'handleSlideshow',
            'handlePrefetching'
        ]);

        // handle view events
        this.on('*:busy', this.onBusy);
        this.on('*:idle', this.onIdle);
        this.on('*:toggleSlideShow', this.toggleSlideShow);
        this.on('*:startSlideShow', this.startSlideShow);
        this.on('*:stopSlideShow', this.stopSlideShow);

        // sharing
        this.on('*:photoCreateLink', this.onPhotoCreateLink);
        this.on('*:photoShareLink', this.onPhotoShareLink);
        this.on('*:photoDisableLink', this.onPhotoDisableLink);
        this.on('*:onPhotoShareLinkClose', this.onPhotoShareLinkClose);

        // download
        this.on('*:download', this.onDownload);

        this.on('share:default', this.onPhotoCreateLink);
        this.on('share:twitter', this.onShareSocial);
        this.on('share:facebook', this.onShareSocial);

        // escape key press
        this.on('galleryView:escape', function () {
            this.navigateToFolder();
        });

        // left key press
        this.on('*:left', function () {
            var previousPhoto = this.getPreviousPhoto();

            if(previousPhoto !== null) {
                this.navigate(this.getPhotoURL(previousPhoto));
            }
        });

        // right key press
        this.on('*:right', function () {
            var nextPhoto = this.getNextPhoto();

            if(nextPhoto !== null) {
                this.navigate(this.getPhotoURL(nextPhoto));
            }

        });


        // if we haven't dispatched because of history change
        this.once('ready', function (e) {
            if(!e.dispatched) {
                this.dispatch();
            }
        });
    },

    /**
    Points the browser to the folder URL

    @method navigateToFolder
    @param {Event} e the event object
    **/

    navigateToFolder: function() {

        var albumURL = this.get('albumURL'),
            httpRe = /^(http|https):\/\//,
            loc = Y.config.win.location;

        Y.log(albumURL, 'note', 'PhotosGalleryApp.navigateToFolder');

        // the app can handle this url
        if(httpRe.test(albumURL)) {
            this.navigate(albumURL);
        }

        // most likely server side invoked, and we need a full reload
        else {
            loc.href = loc.protocol + '//' + loc.host + albumURL;
        }
    },

    /**
    Points browser on next image

    @method navigateToNext
    **/

    navigateToNext: function() {
        var photo = this.getNextPhoto();

        if(photo) {
            this.navigateToPhoto(photo);
        }
    },

    /**
    Points browser on previous image

    @method navigateToPrevious
    **/

    navigateToPrevious: function() {
        var photo = this.getPreviousPhoto();

        if(photo) {
            this.navigateToPhoto(photo);
        }
    },

    /**
    Points browser to supplied photo

    @method navigateToPhoto
    @param {Photo} photo a photo object
    **/
    navigateToPhoto: function (photo) {
        var url = this.getPhotoURL(photo),
            loc = Y.config.win.location;

        // navigate really likes absolute urls.
        url = loc.protocol + '//' + loc.host + url;

        this.navigate(url);
    },

    havePartialAlbum: function (photoList) {
        return photoList.size() < photoList.get('total');
    },

    /**

    <ul>
        <li>`res.photo`: when we request a photo</li>
        <li>`req.query.nodekey` requesting a folder through  `/photos/?nodekey=~/Ubuntu One`</li>
    </ul>

    Calls `next()` only if we have `photos`

    FIXME: when requesting a folder, we do not need `photos` to call `next()`

    @method handleAlbum
    @param {Object} req a request
    @param {Object} res a response
    @param {Function} next the next callback
    **/

    handleAlbum: function (req, res, next) {

        res.photoList = this.get('photoList');
        res.view = this.get('activeView');

        // if there is a slideshow timer, clear it
        if(this.slideShowTimer) {
            Y.config.win.clearTimeout(this.slideShowTimer);
            delete this.slideShowTimer;
        }


        // if there are no photos loaded, fetch them all
        if(res.photoList.size() === 0 || this.havePartialAlbum(res.photoList)) {

            res.photoList.set('pagesize', res.photoList.get('total'));
            res.photoList._galleryModified = true;

            res.photoList.load({ silent: true }, function () {
                res.photoList.set('current_photo', res.photoList.getPhoto(req.params.photo));
                res.photoList.photoSort({ silent: true }, next);
            });

        } else {
            next();
        }

    },

    /**

    @method handlePhoto
    @param {Object} req a request
    @param {Object} res a response
    @param {Function} next the next callback
    **/

    handlePhoto: function (req, res, next) {
        var message = Y.one('.yui3-info');

        res.photo = res.photoList.getPhoto(req.params.photo);
        res.photoList.set('current_photo', res.photo);



        Y.log(res.photo.name, 'note', 'PhotosGalleryApp.handlePhoto');

        // this has been "prefetched"
        this.prefetchPhotos.push(this.get('photoList').indexOf(res.photo));

        // fade out messages if they're shown
        if(message && parseInt(message.getStyle('opacity'), 10) === 1) {
            message.transition({
                duration: 0.3,
                opacity: 0
            });
        }

        // close any share link dialog
        this.fire('shareLinkCloseClick');

        next();

    },

    /**
    Shows folder, which calls for the destruction of this app

    <ul>
        <li>invoked from one-photos, update the one-photos history object</li>
        <li>invoked from the server side (/photos/gallery/photo), `reload()` the `window.location` (we have already called `navigateToFolder`)</li>
    </ul>

    FIXME: pass `folder` argument

    @method closeGallery
    **/

    closeGallery: function () {

        // do not leave any slideShowTimers dangling
        Y.config.win.clearTimeout(this.slideShowTimer);
        delete this.slideShowTimer;

        this.destroy();

    },

    handlePrefetching: function () {

        // since multiples will add to this array, clean up now
        this.prefetchPhotos = Y.Array.unique(this.prefetchPhotos);

        this.prefetch(this.get('photoList').indexOf(this.getCurrentPhoto()));
    },

    handleSlideshow: function (req, res, next) {

        // is the user requesting slideshow?
        var slideshow = (parseInt(req.query.slideshow, 10) === 1);

        // user requested slideshow
        if(slideshow) {
            this.startSlideShow();
        }

        // if the user has not requested slideshow, kill any ongoing slideshow
        if(!slideshow && this.slideShowTimer) {

            Y.config.win.clearTimeout(this.slideShowTimer);
            delete this.slideShowTimer;

        }

        next();
    },

    startSlideShow: function () {

        // are we at the end?
        var photo = this.getCurrentPhoto(),
            photoList = this.get('photoList'),
            atEnd = (photoList.indexOf(photo) === (photoList.size() - 1));

        if(atEnd) {
            // let user know
            Y.Global.fire(ONEINFO, {'message': 'Reached end of slideshow, starting over.'});
        }

        this.get('activeView').photoView.photoWrapper.addClass('playing');

        // create timer, which is an anonymous function bound to this app
        this.slideShowTimer = Y.config.win.setTimeout(Y.bind(function (photoAtTimerInvoke) {

            // if, when timer executes, we're looking at the same image as we did when the timer was invoked,
            // only then navigate
            if(this.getCurrentPhoto().nodekey === photoAtTimerInvoke.nodekey) {

                Y.log('navigateToNext', 'note', 'PhotosGalleryApp.startSlideShow');

                // navigate to next photo and set slideshow=1
                // if at end, navigate to first photo
                if(atEnd) {
                    this.navigateToPhoto(photoList.getPhoto(0));
                } else {
                    this.navigateToNext();
                }

            }
        }, this, (this.getCurrentPhoto())), SLIDESHOWINTERVAL);
    },

    stopSlideShow: function () {
        this.get('activeView').photoView.photoWrapper.removeClass('playing');
    },

    toggleSlideShow: function () {

        if(this.get('photoList').get('total') < 2) {
            return;
        }

        if(this.slideShowPlaying()) {
            this.updateQueryParameterAndNavigate(this.getQueryStringAsObject(), 'slideshow', 0);
            this.stopSlideShow();
        } else {
            this.updateQueryParameterAndNavigate(this.getQueryStringAsObject(), 'slideshow', 1);
        }
    },

    slideShowPlaying: function () {
        return (this.getCurrentPhotoNode().get('parentNode').hasClass('playing'));
    },

    /**

    adds `loading` class to container

    @method onBusy
    **/

    onBusy: function() {
        this.get('container').addClass('loading');
    },

    /**

    removes `loading` class from container

    @method onIdle
    **/

    onIdle: function () {
        this.get('container').removeClass('loading');
    },

    /**
    @method getCurrentPhotoKey
    @return {String} ret current photo nodekey
    **/

    getCurrentPhotoKey: function () {
        var photoNode = this.getCurrentPhotoNode();

        return photoNode ? photoNode.getData('nodekey') : null;
    },

    getCurrentPhotoNode: function () {
        return this.get('activeView').get('container').one('.photo img');
    },

    /**

    Note that it's a Lazy List, so not a photo model

    @method getCurrentPhoto
    @param revive if true, really return photo model
    @return {Photo} current photo object
    **/

    getCurrentPhoto: function (revive) {
        var currentPhotoKey = this.getCurrentPhotoKey(),
            photo = this.get('photoList').getPhoto(currentPhotoKey);

        return revive ? this.get('photoList').revive(photo) : photo;
    },

    /**

    @method getNextPhoto
    @return {Photo} photo
    **/

    getNextPhoto: function () {

        var photo = this.getCurrentPhoto(),
            photoList = this.get('photoList'),
            nextPhotoIndex = photoList.indexOf(photo) + 1;

        if(nextPhotoIndex > photoList.size() - 1) {
            return null;
        }

        return photoList.getPhoto(nextPhotoIndex);
    },

    /**

    @method getPreviousPhoto
    @return {Photo} photo a photo object
    **/

    getPreviousPhoto: function () {

        var photoList = this.get('photoList'),
            currentPhotoIndex = photoList.indexOf(this.getCurrentPhoto()),
            previousPhotoIndex = currentPhotoIndex - 1;

        if(previousPhotoIndex < 0) {
            return null;
        }

        return photoList.getPhoto(previousPhotoIndex);
    },

    /**

    <p>Creates URLs based on the photoURLTemplate</p>

    note that they are relative

    @method getPhotoURL
    @param {Photo} photo from which to extract data to build URL
    @return {String} url to a photo
    **/

    getPhotoURL: function (photo, queryParams) {
        var params = this.getQueryString();

        if(queryParams) {
            if(params) {
                params = params + '&';
            }
            params = params + Y.QueryString.stringify(queryParams);
        }

        if(!/\?/.test(params) && params) {
            params = '?' + params;
        }

        return Y.substitute(
            this.photoURLTemplate,
            {
                photo_nodekey: photo.nodekey,
                photo_name: photo.name,
                params: params,
                path: this.get('photoList').get('path')
            },
            function (k, v) {
                // encode name and path as URI
                // as long as it's not already encoded
                if(k === 'photo_name' || k === 'path') {
                    if(decodeURI(v) !== v) {
                        v = encodeURI(v);
                    }
                }
                return v;
            }
        );
    },

    prefetchPhotos: [],

    /**

    Prefetches the next `n` photos from `currentIndex`

    @method prefetch
    @param {Number} currentIndex the index of the photo being shown
    @param {Number} n how many images to preload
    @param {Number} minimum the minimum amount of images to return
    **/

    prefetch: function (currentIndex, n, minimum) {
        var amount = n || 3,
            from = currentIndex + 1,
            photos = this.get('photoList').toArray(),
            toBePrefetched = [],
            i,
            httpRe = /^(http|https):\/\//;

        // prepare a list to be prefetched
        for(i=amount; i--;) {

            // out of bounds
            if(!photos[from + i]) {
                continue;
            }

            // if we haven't already prefetched this photo, add queue it for prefetching
            if(Y.Array.indexOf(this.prefetchPhotos, from + i ) === -1) {
                toBePrefetched.push(from + i);
            }

        }

        // if this list of prepared photos to be prefetched is large enough, prefetch the photos
        if(minimum && toBePrefetched.length < minimum) {
            return;
        }

        toBePrefetched.reverse();

        Y.Array.each(toBePrefetched, function (item) {

            var img = new Image(),
                src;

            if(this.get('photoList').get('public_list')) {
                src = Y.substitute(
                    '{public_thumb_base_url}/1000x1000/{name}',
                    photos[item],
                    function (k, v) {
                        // encode what the user can control, the name
                        // public_thumb_base_url is safe, coming from us
                        if(k==='name') {
                            v = encodeURIComponent(v);
                        }
                        return v;
                    }
                );

            } else {
                // thumbnail_url is coming from us
                // safe
                src = Y.substitute(
                    '{thumbnail_url}/1000x1000/',
                    photos[item]
                );
            }

            // ensure that the img src starts with http
            // and is not empty
            if(httpRe.test(src) && src !== "") {
                img.src = src;
                this.prefetchPhotos.push(item);
            }

        }, this);

    },

    afterCurrentPhotoChange: function () {
        this.get('activeView').render(this);
    },

    afterPhotosReset: function () {
        this.get('activeView').render(this);
    },

    getShareLink: function(photo, success, failure, type) {
        var request = '/api/photos/v1/photo_make_public';

        Y.io(request, {
            method: 'POST',
            data: {
                'public': 'true',
                'csrfmiddlewaretoken': Y.one('[name=csrfmiddlewaretoken]').get('value'),
                'nodekey': photo.nodekey
            },
            on: {
                success: Y.bind(success, this),
                failure: Y.bind(failure, this)
            },
            "arguments": {
                type: type
            }
        });
    },

    /**
    Somebody has requested a public link to the photo

    Makes a post to the photo_make_public api

    @method onPhotoCreateLink
    **/
    onPhotoCreateLink: function () {
        var photo = this.getCurrentPhoto();

        Y.log(photo.name, 'note', 'PhotosGalleryApp.onPhotoCreateLink');

        this.fire('busy');

        this.getShareLink(photo, this.onPhotoCreateLinkSuccess, this.onPhotoCreateLinkError);
    },

    /**
    Creating a link was successful

    Will re-render actions and photo meta as well as fire onPhotoShareLink

    @method onPhotoCreateLinkSuccess
    @param {String} id id
    @param {Object} res the response xmlhttpreq
    **/
    onPhotoCreateLinkSuccess: function (id, res) {
        Y.log('Success', 'note', 'PhotosGalleryApp.onPhotoCreateLinkSuccess');

        var response = Y.JSON.parse(res.responseText),
            photo = this.getCurrentPhoto();

        // TODO cache the url in an attr
        photo.public_url = response.public_url ;

        this.get('activeView').actionsView.render(photo);
        this.get('activeView').metaView.render(photo);

        this.fire('photoShareLink');

        this.fire('idle');
    },

    /**
    Handler for when creating a link has failed. Gives feedback.

    @method onPhotoCreateLinkError
    @param {Event} e event
    **/
    onPhotoCreateLinkError: function (e) {
        Y.log(e, 'error', 'PhotosGalleryApp.onPhotoCreateLinkError');

        this.fire('idle');

        Y.Global.fire(ONEERROR, {msg: 'Could not create the link. Please try again later'});
    },

    /**
    Handler for whenever something has requested sharing the photo public_url
    By default it spawns a Y.Panel widget, but this can be turned off

    @method onPhotoShareLink
    @param {Event} e event
    @param {Boolean} [e.createShareLinkWidget=true] createShareLinkWidget if truthy, it creates a modal overlay
    **/
    onPhotoShareLink: function (e) {
        var createShareLinkWidget = true,
            publicURL = this.getCurrentPhoto().public_url;

        if(typeof e.createShareLinkWidget !== 'undefined') {
            createShareLinkWidget = e.createShareLinkWidget;
        }

        if (createShareLinkWidget) {

            // TODO: move this into the view
            if(!this.shareLinkWidget ) {

                this.createShareLinkWidget();

            } else {
                this.shareLinkWidget.show();
            }
            // update the url
            // make it easier for the user to copy by pre selecting link
            this.shareLinkWidget.getStdModNode(Y.WidgetStdMod.BODY, true).one('input').setAttribute('value', publicURL).select();
        }

    },

    /**
     * create a Panel to show the created link
     */
    createShareLinkWidget: function() {

        this.shareLinkWidget = new Y.Panel({
            headerContent: '<h3>Share link to photo</h3>',
            bodyContent: '<div><p>Copy the link to share this photo:<input class="u1-input-share"></input></div>',
            width: 400,
            centered: true,
            render: true,
            visible: true,
            modal: true,
            zIndex: 10000,
            buttons: {
                header: ['close'],
                footer: [{
                    classNames: 'cta',
                    value: 'Done',
                    action: function(e) {
                        e.preventDefault();
                        this.hide();
                    }
                },{
                    classNames: ['cta', 'secondary'],
                    value: 'Disable Link',
                    action: function(e) {
                        e.preventDefault();
                        this.fire('photoDisableLink');
                        this.hide();
                    }
                }
                ]
            },
            plugins: [
                Y.U1Plugin.PanelAnim
            ]
        });

        // ah, pubsub, i love thee
        this.shareLinkWidget.addTarget(this);

        // don't want the panel getting out of position
        Y.on("resize", Y.bind(function() {
            this.shareLinkWidget.centered();
        }, this));
    },

    /**
     * onShareSocial - Handler for user initiated sharing of resource
     * @method onShareSocial
     * @param {Event} e custom event containing type of share
     */
    onShareSocial: function(e) {
        var photo = this.getCurrentPhoto(),
            type = e.type;

        this.getShareLink(photo, this.onShareSocialSuccess, this.onShareSocialError, type);
    },

    /**
     * onShareSocialSuccess callback when share url created
     * @param {String} id id
     * @param {Object} res the response xmlhttpreq
     * @param {Object} args arguments bound to the calling function,
     * contains the type of the share (Twitter, Facebook, G+, etc)
     */
    onShareSocialSuccess: function(id, res, args) {
        var response = Y.JSON.parse(res.responseText),
            photo = this.getCurrentPhoto(),
            photo_url = photo.display_url,
            link = response.public_url,
            popup,
            type = args.type.replace('share:', '');

        photo.public_url = link;

        this.get('activeView').actionsView.render(photo);
        this.get('activeView').metaView.render(photo);

        popup = new Y.U1.util.Share(link, photo_url);
        popup.setUrl(type).open(null, null, type);

    },

    /**
     * onShareSocialError: Handler for io error
     */
    onShareSocialError: function(type) {
        Y.log('Y.io failed on '+type);
    },


    /**
    Handler for when something has requested the share link dialog to close

    @method onPhotoShareLinkClose
    @param {Event} e event
    **/
    onPhotoShareLinkClose: function () {
        if(this.shareLinkWidget) {
            this.shareLinkWidget.hide();
        }
    },

    /**
    Handler for when something has requested a photo to become private

    Posts to photo_make_public

    @method onPhotoDisableLink
    **/
    onPhotoDisableLink: function () {
        var photo = this.getCurrentPhoto(),
            request = '/api/photos/v1/photo_make_public';

        Y.log(photo.name, 'note', 'PhotosGalleryApp.onPhotoDisableLink');

        this.fire('busy');

        Y.io(request, {
            method: 'POST',
            data: {
                'public': 'false',
                'csrfmiddlewaretoken': Y.one('[name=csrfmiddlewaretoken]').get('value'),
                'nodekey': photo.nodekey
            },
            on: {
                success: Y.bind(this.onPhotoDisableLinkSuccess, this),
                failure: Y.bind(this.onPhotoDisableLinkSuccess, this)
            }
        });
    },

    /**
    If making the photo private was successful, this handler is called

    Re-renders actions and meta, fires events requesting link share dialog to close.

    Gives feedback

    @method onPhotoDisableLinkSuccess
    **/
    onPhotoDisableLinkSuccess: function () {
        var photo = this.getCurrentPhoto();

        photo.public_url = null;

        Y.log('Success', 'note', 'PhotosGalleryApp.onPhotoDisableLinkSuccess');

        this.get('activeView').actionsView.render(photo);
        this.get('activeView').metaView.render(photo);

        this.fire('shareLinkCloseClick');

        this.fire('idle');

        Y.Global.fire(ONEINFO, {'message': 'The link to this photo is now disabled'});
    },

    /**
    Handler for when making photo private failed

    Gives feedback to user

    @method onPhotoDisableLinkError
    **/
    onPhotoDisableLinkError: function (e) {
        Y.log(e, 'error', 'PhotosGalleryApp.onPhotoDisableLinkSuccess');

        Y.Global.fire(ONEERROR, {msg: 'Could not disable the link. Please try again later'});

        this.fire('idle');
    },

    onDownload: function () {
        Y.config.win.location.href = this.getCurrentPhoto().display_url;
    }

}, {
    ATTRS: {

        // do not handle stuff exiting the gallery, like the close button and u1 logo
        linkSelector: '.preview-wrapper a, .gallery-footer a',

        /**
        @attribute routes
        @type Array

        **/
        routes: {}
    }
});

Y.namespace('Photos').GalleryApp = PhotosGalleryApp;


}, '@VERSION@', {
    "requires": [
        "attribute",
        "base-build",
        "app",
        "escape",
        "node",
        "photos-common",
        "photos-gallery-app-view",
        "photos-gallery-photo-view",
        "photos-gallery-pager-view",
        "photos-gallery-actions-view",
        "photos-gallery-meta-view",
        "photos-photo-list-model",
        "photos-photo-model",
        "photos-io",
        "panel",
        "plugin-panel-anim",
        "event-key",
        "substitute",
        "transition",
        "util-share"
    ]
});
