YUI.add('photos-gallery-photo-view', function (Y, NAME) {

"use strict";
var TransitionEffect = {
        Out: {
            opacity: 0,
            duration: 0.1
        },
        // In has currently no effect
        In: {
            opacity: 1,
            duration: 0.3
        }
    },

    PHOTOWRAPPER = '.photo-wrapper',

    GalleryPhotoView;

/**
galleryPhoto is the view that represents the photo we're currently showing the user

@class GalleryPhoto
@constructor
@extends View

**/

GalleryPhotoView = Y.Base.create('galleryPhoto', Y.View, [], {

    /**
    @property template
    @type Object

    **/

    template: Y.one('#gallery-photo-template').getContent(),

    events: {
        '.photo-wrapper': {
            mouseover: 'onPhotoMouseOn',
            mouseout: 'onPhotoMouseOff',
            click: function () {
                this.fire('toggleSlideShow');
            }
        }
    },


    /**
    Caching the wrapper around the photo
    @property photoWrapper
    @type Node
    **/
    photoWrapper: null,

    initializer: function () {

        this.photoWrapper = this.get('container').one(PHOTOWRAPPER);

        if(this.get('photoList').get('total') < 2) {
            this.detachAll(PHOTOWRAPPER);
        }

    },

    /**

    Renders a photo

    @method render
    @chainable
    @return {GalleryPhotoView} GalleryPhotoView
    **/

    render: function () {
        var container = this.get('container').one('.photo-wrapper'),
            photo = this.get('photoList').get('current_photo'),
            oldImage = container.one('img'),
            newImage;

        // no need to render the same image twice
        if(oldImage.getData('nodekey') === photo.nodekey) {
            return;
        }

        newImage = Y.Node.create(Y.substitute(this.template, {
            name: photo.name,
            nodekey: photo.nodekey,
            thumbnail_url: photo.thumbnail_url,
            public_thumb_base_url: photo.public_thumb_base_url
        }, function(k,v){return Y.Escape.html(v);}));

        this.fire('busy');

        // fade out early to prevent frankenimages
        // at worst there will be a frankenimage made of the old (at 20% opacity) and the new
        oldImage.transition({
            opacity: 0.2,
            duration: 0.3
        });

        // since we're fading it in, before prepending, make invisible
        newImage.setStyle('opacity', 0);
        container.prepend(newImage);

        // fade in new image
        newImage.transition(TransitionEffect.In);

        // fade out old image completely and then remove it
        oldImage.transition(TransitionEffect.Out, function () {
            oldImage.remove();
        });

        // on load fires 'idle' and calls next
        newImage.on('load', this.onImageLoad, this, photo);
        newImage.on('error', this.onImageLoadFailed, this, photo);

        return this;
    },

    /**
    Executed whenever a new photo is loaded/ready

    <ul>
        <li>Fires the `photoLoad` event, which bubbles and is caught by the {{#crossLink "PhotosGalleryView"}}{{/crossLink}}</li>
        <li>Fires idle</li>
        <li>calls next route callback</li>
        <li>if there is a wrapper, then it adds some classes to it</li>
        <li>there is a case where the wrapper will not exist due to the user calling next image faster than the image loads, then we do nothing</li>
    </ul>

    @method onImageLoad
    @param {Event} e the event object
    @param {Photo} photo the currently being rendered photo object
    @param {HTMLElement|Node} oldImage the old image node
    @param {Function} next the next (route) callback
    **/

    onImageLoad: function (e) {
        var wrapper = e.target.get('parentNode');

        this.fire('idle');

        // add the appropriate classes to the wrapper
        // if next image is called before this image is loaded, the wrapper will not exist, so do not try to operate on it
        if(wrapper) {
            e.target.get('parentNode').addClass('image-loaded').removeClass('image-not-loaded');
        }
    },

    /**
    Executed whenever a photo fails to load (corrupted in any way, deleted since we opened the gallery)

    @method onImageLoadFailed
    @param {Event} e the event object
    @param {Photo} photo the currently being rendered photo object
    @param {HTMLElement|Node} oldImage the old image node
    @param {Function} next the next (route) callback
    **/

    onImageLoadFailed: function (e) {
        Y.log('Failed to load image', 'error', 'GalleryPhoto.onImageLoadFailed');

        this.fire('idle');

        e.target.get('parentNode').addClass('image-not-loaded').removeClass('image-loaded');
    },


    /**

    on photo mouse on

    @method onPhotoMouseOn
    **/

    onPhotoMouseOn: function () {
        var photoListSize = this.get('photoList').get('total');
        if (photoListSize && photoListSize > 1) {
            this.photoWrapper.addClass('hover');
        }
    },

    /**

    on photo mouse off

    @method onPhotoMouseOff
    **/

    onPhotoMouseOff: function () {
        var photoListSize = this.get('photoList').get('total');
        if (photoListSize && photoListSize > 1) {
            this.photoWrapper.removeClass('hover');
        }
    }

}, {
    ATTRS: {

        container: {
            value: '.preview-photo'
        }

    }
});

Y.namespace('Photos').GalleryPhotoView = GalleryPhotoView;


}, '@VERSION@', {"requires": ["view"]});
