YUI.add('plugin-jit-bar', function (Y, NAME) {

 "use strict";

var JITBar = function (config) {
    config = (config) ? Y.merge(config) : {};
    config.node = config.host;
    JITBar.superclass.constructor.apply(this, arguments);
};

JITBar.NAME = 'JITBarPlugin';
JITBar.NS = 'JITBar';

JITBar.ATTRS = {

    proxy: {
        value: null,
        setter: function (val) {

            if(!val) { return val; }

            var host = this.get('host');

            val
                .addClass('jit-proxy')
                .insertBefore(val, host);

            host.setStyle('visibility', 'hidden');
            val.setStyle('visibility', 'visible');

            this.animation.set('node', val);

            return val;
        }
    },

    // how many pixels we will subtract from the original font size
    // TODO: fontMinSize?
    fontSizeSubtrahend: {
        value: 8
    },

    // virtual keyboard and position: fixed on ios is broken
    // if the jit bar has an element invoking the virtual keyboard
    // this fix will scroll to the top and set the jit bar's position
    // to absolute
    virtualKeyboardFix: {
        value: Y.UA.ios > 0
    },

    scrollingParent: {
        value: null,
        lazyAdd: true,
        setter: function () {

            var events = ['scroll', 'touchmove'],
                val = Y.one('body');

            Y.on(events, Y.bind(this.redraw, this));

            val.plug(Y.Plugin.ScrollInfo);

            return val;

        }
    },

    // in percent the amount we've reduced the bar
    // 0 is no reduction, 100 is full reduction
    reduction: {
        value: 0,
        validator: function (val) {
            return Y.Lang.isNumber(val) && (val <= 100) && (val >= 0);
        },
        setter: function (val) {
            return Math.round(val / 10) * 10;
        }
    }

};

Y.extend(JITBar, Y.Plugin.Base, {

    animation: new Y.Anim({
        duration: 0.1,
        easing: 'easeBoth'
    }),

    initializer: function() {

        this.set('proxy', this.get('host').cloneNode(true));

        this.bindJITEvents();

    },

    bindJITEvents: function () {

        this.get('scrollingParent').scrollInfo.on([
            'scrollToBottom',
            'scrollUp',
            'scrollDown'
        ], this.redraw, this);

        Y.config.win.addEventListener('pageshow', Y.bind(this.redraw, this), true);
        Y.config.win.addEventListener('orientationchange', Y.bind(this.redraw, this), true);
        Y.on('windowresize', Y.bind(this.redraw, this));

        this.after('reductionChange', this._reductionChange);

        Y.delegate('click', this.elevate, this.get('host').ancestor(), '.jit-proxy', this);


        if(this.get('virtualKeyboardFix')) {
            this.useVirtualkeyboardFix();
        }
    },

    useVirtualkeyboardFix: function () {
        var p = this.get('proxy');
        p.all('input').on('focus', this._childInputFocus, this);
        p.all('input').on('blur', this._childInputBlur, this);

    },

    _childInputFocus: function () {
        this.get('proxy').addClass('jit-fix-fixed');
        this.get('host').scrollIntoView();
    },

    _childInputBlur: function () {
        this.get('proxy').removeClass('jit-fix-fixed');
    },

    redraw: function (e) {

        var scrolledPastBar = this.getScrolled() > this.getOriginalHeight(),
            isScrollUp = this.get('scrollingParent').scrollInfo.getScrollInfo().isScrollUp;

        // break lock if above bar
        if(!scrolledPastBar) {
            this.modifyAttr('reduction', { readOnly: false });
        }

        if(scrolledPastBar && Y.Lang.isBoolean(e.isScrollUp)) {
            this._scrollInfoScroll(e);
        } else if (!isScrollUp) {
            this._scroll(e);
        }

    },

    refreshDimensions: function () {
        delete this._minHeight;
        delete this._originalHeight;
        delete this._originalFontSize;
    },

    _scroll: function () {

        this.set('reduction', this.scrollToReduction());

    },

    _scrollInfoScroll: function (e) {

        if(e.isScrollDown) {
            this.reduce();
        }

        if(e.isScrollUp || e.atBottom) {
            this.elevate();
        }

    },

    // TODO: maybe move some of this logic to the getter or setter of scrollingParent
    getScrolled: function () {
        return this.get('scrollingParent').scrollInfo.getScrollInfo().scrollTop;
    },

    // translate the scrolling to how much transformation we need
    scrollToReduction: function () {
        var reducableHeight = this.getOriginalHeight() - this.getMinHeight(),
            amount = (100 / reducableHeight) * this.getScrolled();

        return Math.floor(Math.min(100, amount));
    },

    getMinHeight: function () {
        if(typeof this._minHeight === 'undefined') {
            this._minHeight = parseInt(this.get('host').getComputedStyle('minHeight'), 10);
        }
        return this._minHeight;
    },

    getOriginalHeight: function () {
        if(typeof this._originalHeight === 'undefined') {
            this._originalHeight = parseInt(this.get('host').getComputedStyle('height'), 10);
        }
        return this._originalHeight;
    },

    getReducedHeight: function (amount) {
        var reducableHeight = this.getOriginalHeight() - this.getMinHeight();
        return this.getOriginalHeight() - ( ( reducableHeight / 100 ) * amount);
    },

    getOriginalFontSize: function () {
        if(typeof this._originalFontSize === 'undefined') {
            this._originalFontSize = parseInt(this.get('host').getComputedStyle('fontSize'), 10);
        }
        return this._originalFontSize;
    },

    getReducedFontSize: function (amount) {
        //return this.getOriginalFontSize() - ( (100 / this.get('fontSizeSubtrahend')) * amount);
        return this.getOriginalFontSize() - ( (this.get('fontSizeSubtrahend') / 100) * amount );
    },

    animate: function (amount) {

        if(this.animation.get('running')) {
            this.animation.stop();
        }

        this.animation.set('to', {
            fontSize: this.getReducedFontSize(amount) + 'px',
            height: this.getReducedHeight(amount)
        });

        this.animation.run();

    },

    reduce: function () {
        this.set('reduction', 100);
    },

    elevate: function () {
        this.set('reduction', 0);
    },

    setCssClasses: function (amount) {

        var p = this.get('proxy'),
            reduced = 'jit-reduced',
            elevated = 'jit-elevated';

        if(amount >= 60) {
            p.addClass(reduced);
        } else {
            if(p.hasClass(reduced)) {
                p.removeClass(reduced);
            }
        }

        if (amount <= 40) {
            p.addClass(elevated);
        } else {
            if(p.hasClass(elevated)) {
                p.removeClass(elevated);
            }
        }

    },

    _reductionChange: function (e) {

        if(e.prevVal === e.newVal) {
            return;
        }

        this.setCssClasses(e.newVal);

        this.animate(e.newVal);

        // if we've reduced it completely, do not allow elevation in n seconds
        if(e.newVal === 100) {

            // lock reduction attr
            this.modifyAttr(e.attrName, {
                readOnly: true
            });

            Y.later(3000, this, function () {
                this.modifyAttr(e.attrName, {
                    readOnly: false
                });
            });

        }

    }

});

Y.namespace('U1Plugin');
Y.U1Plugin.JITBar = JITBar;


}, '@VERSION@', {"requires": ["anim", "node", "node-scroll-info", "plugin"]});
