YUI.add('uploader', function (Y, NAME) {

 "use strict";

/**
* Extends uploader-html5 with a frontend and functionality.
* Features:
*   - `onbeforeunload` event handled and will ask user if there are uploads pending or in progress
*   - fires the `fullycomplete`/`partialcomplete`/`incomplete`, which will be more accurate than `alluploadscomplete`
        * since it considers failed uploads or if all uploads are cancelled
*   - a `status` attribute will reflect the current status of the uploader
*   - a `dropAreaStatus` attribute that will reflect the current status of the drag and drop area
*   - adds a `status` attribute to files, keeping track of it's status
*   - duck punches uploader-html5 in the following ways:
        1. calculates `percentLoaded` for the `totaluploadprogress` event
        2. make the queue `uploadcacel` fire `uploadcancel`
        3. recalculates bytes uploaded/total for active uploader-queue when cancelling a file
*   - API for checking state of uploader
*
* @module one-uploader
*/
var Uploader = Y.Base.create('oneUploader', Y.UploaderHTML5, [], {

    initializer: function () {

        Y.before(this._bindUploaderEvents, this, 'bindUI');

        this._setUnload();

        /**
        * Fires when the UI has no more uploads left
        * This is different than the queue reporting it is done,
        * since the queue can skip files
        *
        * @event fullycomplete
        */
        this.publish('fullycomplete');

        /**
        * Fires when the UI has one or more uploads left
        * Exists because of queues reporting done even if uploads failed
        *
        * @event fullycomplete
        */
        this.publish('partialcomplete');

        /**
        * Fires when all uploads were cancelled, or failed
        *
        * @event incomplete
        */
        this.publish('incomplete');

        /**
        * Fires when all files were cancelled
        *
        * @event alluploadscancelled
        */
        this.publish('alluploadcancelled');

        this.set('status.text', this.get('strings.uploads_empty'));
        this.set('dropAreaStatus', this.get('strings.drop_ready'));
        this.get('contentBox').addClass('uploader');

    },

    destructor : function() {

        this._unsetUnload();

    },

    _setUnload: function () {

        if(!Y.config.win.onbeforeunload) {
            Y.config.win.onbeforeunload = Y.bind(this._beforeUnload, this);
            Y.config.win.onbeforeunload._owner = this.name;
        }

    },

    _unsetUnload: function () {

        if(Y.config.win.onbeforeunload && Y.config.win.onbeforeunload._owner === this.name) {
            Y.config.win.onbeforeunload = null;
        }

    },

     /**
    * return message to user if there's something it should know before unloading
    *
    * @method _beforeUnload
    * @private
    */
    _beforeUnload: function (e) {
        var msg = this.getUnloadMessage();
        if(msg) {
            e.returnValue = msg;
            return msg;
        }
    },

    /**
    * Binds all upload events, as well as some UI events
    *
    * @method _bindUploaderEvents
    */
    _bindUploaderEvents: function () {

        this._bindQueueEvents();
        this._bindFilesEvents();
        this._bindDD();

        this.before('fileListChange',           this._fileListChange);
        this.after('dragAndDropAreaChange',     this._bindDD);

    },

     /**
    * bind events for queue
    *
    * @method _bindQueueEvents
    * @private
    */
    _bindQueueEvents: function () {

        this.on('totaluploadprogress',              this._totalUploadProgress);
        this.after('uploaderror',                   this._errorCheck);
        this.after('filecancel',                    this._cancelCheck);
        this.on('alluploadscomplete',               this._postCompleteCheck);

    },

     /**
    * bind events for files
    *
    * @method _bindFilesEvents
    * @private
    */
    _bindFilesEvents: function () {

        this.on('uploadstart',                      this._uploaderStart);
        this.on('uploadprogress',                   this._uploadProgress);
        this.on('uploadcomplete',                   this._uploadComplete);
        this.on('uploaderror',                      this._uploadError);
        this.on('filecancel',                       this._uploadCancel);
        this.on('fileuploadstart',                  this._uploadStart);

    },

     /**
    * handler for when a drag starts
    * @method startDrag
    */
    startDrag: function () {

        this.set('dropAreaStatus', this.get('strings.drop_pending'));
        this.get('contentBox').addClass('uploader-drop-pending');

    },

     /**
    * handler for when a drag ends
    * @method endDrag
    */
    endDrag: function () {

        this.set('dropAreaStatus', this.get('strings.drop_ready'));
        this.get('contentBox').removeClass('uploader-drop-pending');

    },

    /**
    * bind drag/drop events
    *
    * @method _bindDD
    * @private
    */
    _bindDD: function (event) {
        var ev = event || {prevVal: null},
            ddArea = this.get('dragAndDropArea');

        if (ev.prevVal !== null) {
            this.detach('drop');
            this.detach('dragenter');
            this.detach('dragover');
            this.detach('dragleave');
        }

        if (ddArea !== null) {
            this.on(['dragenter', 'dragover'], this.startDrag);
            this.on(['dragleave', 'drop'], this.endDrag);
        }

    },

    /**
    * checks the state of uploader after a file fails
    *
    * @method _errorCheck
    * @private
    */
    _errorCheck: function () {

        if(!this.haveProgressingFiles()) {

            if(this.haveServerFiles()) {

                this.partialComplete();

            } else {

                this.incomplete();

            }

        }

    },

    _cancelCheck: function () {

        if(this.containsOnlyState(Y.U1.Widgets.Uploader.CANCELLED)) {

                // the only state we have is cancelled
                this.cancelAllUploads();

        } else {

            if(!this.haveProgressingFiles() && !this.havePendingFiles()) {

                if(this.haveServerFiles()) {
                    // we have uploaded files, and no files in progress
                    this.partialComplete();

                } else {
                    // only local files, no progressing files
                    this.incomplete();

                }

            }

        }

    },

    /**
    * checks the state of uploader after a file or queue completes
    *
    * @method _postCompleteCheck
    * @private
    */
    _postCompleteCheck: function () {

        var complete    = this.haveServerFiles(),
            failures    = this.haveFailedFiles(),
            cancelled   = this.haveCancelledFiles();

        if(this.containsOnlyState(Y.U1.Widgets.Uploader.COMPLETE)) {

            this.fullyComplete();

        } else if (complete && (failures || cancelled)) {

            this.partialComplete();

        } else if (!complete) {

            this.incomplete();

        }

    },

    /**
    * Called when all files fully complete
    *
    * @method fullyComplete
    */
    fullyComplete: function () {

        this.set('status.text', this.get('strings.fullycomplete'));
        this.fire('fullycomplete');

    },

    /**
    * Called when partially complete (queue done, some failed or some cancelled files)
    *
    * @method partialComplete
    */
    partialComplete: function () {

        this.set('status.text', this.get('strings.partialcomplete'));
        this.fire('partialcomplete');

    },

    /**
    * Called when no files were uploaded but queue reported done
    *
    * @method incomplete
    */
    incomplete: function () {

        this.set('status.text', this.get('strings.incomplete'));
        this.fire('incomplete');

    },

    _uploaderStart: function () {

        this.set('enabled', false);

    },

    _uploadProgress: function (e) {

        e.file.set('status', Y.U1.Widgets.Uploader.UPLOADING);

    },

    _uploadComplete: function (e) {

        e.file.set('status', Y.U1.Widgets.Uploader.COMPLETE);
        e.file.modifyAttr('status', {
            readOnly: true
        });

    },

    _uploadError: function (e) {

        e.file.set('status', Y.U1.Widgets.Uploader.FAILED);

    },

    _uploadCancel: function (e) {

        e.file.set('status', Y.U1.Widgets.Uploader.CANCELLED);

    },

    _uploadStart: function (e) {

        e.file.set('status', Y.U1.Widgets.Uploader.STARTED);

    },

    /**
    * sets status attribute when file list is changed
    *
    * @method _filesChangeCheck
    * @private
    */
    _fileListChange: function (e) {

        var totals = this.getFilesTotals(e.newVal);

        this.set('status.count', totals.count);
        this.set('status.size', totals.size);

        Y.each(e.newVal, function (file) {
            if(typeof file.get('status') === 'undefined') {
                file.addAttr('status', {
                    value: Y.U1.Widgets.Uploader.PENDING
                });
            }
        }, this);

        if(this.containsOnlyState(Y.U1.Widgets.Uploader.PENDING, e.newVal)) {

            this.set('status.text', this.get('strings.uploads_pending'));

        }

        if (!this.queue && e.newVal.length === 0) {

            this.set('status.text', this.get('strings.uploads_empty'));

        }

    },

    /**
    * return message on unload event based on uploader status
    *
    * @method getUnloadMessage
    */
    getUnloadMessage: function () {
        var msg;

        if(this.haveProgressingFiles()) {

            msg = Y.Lang.sub(this.get('strings.cancel_remaining'), { count: this.queue.queuedFiles.length});

        } else if(this.haveLocalFiles()) {

            msg = Y.Lang.sub(this.get('strings.cancel_pending'), { count: this.getLocalFiles().length });

        }

        return msg;
    },

    /**
    * Main path for destroying the uploader
    * Will ask the user nicely what to do
    * if there are uploads remaining/ongoing etc
    *
    * @method _confirmCancel
    */
    _confirmCancel: function () {

        var msg = this.getUnloadMessage();

        if(this.haveProgressingFiles()) {

            if (Y.config.win.confirm(msg)) {

                // make sure we nuke the xhrs
                this.cancelAllUploads();

                this.destroy();
            }

        } else if(this.haveLocalFiles()) {

            if (Y.config.win.confirm(msg)) {
                this.destroy();
            }

        } else {
            this.destroy();
        }

    },

    _totalUploadProgress: function (event) {

        // XXX: if we have uploaded bytes and totalbytes, duck punch event
        // https://github.com/yui/yui3/issues/1176
        if(event.bytesLoaded && event.bytesTotal) {
            event.percentLoaded = Math.min(100, Math.round(10000*event.bytesLoaded/event.bytesTotal) / 100);
        }

        var p = event.percentLoaded;

        if(p) {

            // for numerical values
            if(!isNaN(parseFloat(p)) && isFinite(p)) {
                this.set('status.percent', p);
                this.set('status.text', this.get('strings.totaluploadprogress'));
            } else {
                this.set('status.text', this.get('strings.unknownprogress'));
            }

        }

    },

    /**
    * Starts uploading all files
    *
    * @method start
    * @public
    */
    start: function () {
        this.uploadAll();

        // duck punch broken uploader (uploader-queue)
        this.queue.on('uploadcacel', function () {
            this.fire('uploadcancel');
        }, this);

    },

    // main cancel path, used in X-buttons and Cancel-buttons
    cancel: function () {

        this._confirmCancel();

    },

    /**
    * Cancels queue and fires the incomplete event
    *
    * @method cancelAllUploads
    * @public
    */
    cancelAllUploads: function () {

        if (this.queue) {
            this.queue.cancelUpload();
        }
        this.fire('alluploadcancelled');

    },

    cancelFile: function (fileToCancel) {

        // set fileList to array without file to be deleted
        this.set('fileList', Y.Array.filter(this.get('fileList'), function (file) {
            return file.get('id') !== fileToCancel.get('id');
        }));

        // if a queue is active
        if (this.queue) {

            // cancel upload, remove from queue
            this.queue.cancelUpload(fileToCancel);

            // the next two lines should be handled by YUI Uploader Queue, but is not
            // monkeypatching in the mean time
            // reduce totalBytes
            this.queue.totalBytes -= fileToCancel.get('size');

            // remove the file's uploaded bytes from uploaded bytes total
            this.queue.totalBytesUploaded = Math.max(0, this.queue.totalBytesUploaded - fileToCancel.get('bytesUploaded'));

        }

        this.fire('filecancel', {
            file: fileToCancel
        });
    },

    retryFile: function (file) {

        var q = this.queue;

        // if no queue at all, use uploadThese
        if(!q) {
            this.uploadThese([file], this.get('url'));
        } else {
            // if the file was cancelled by error, re-register
            if(!q.currentFiles.hasOwnProperty(file.get('id'))) {
                q._registerUpload(file);
            }

            q.forceReupload(file);
        }

    },

    getFile: function (id) {

        return Y.Array.find(this.get('fileList'), function (file) {
            return file.get('id') === id;
        });

    },

    getFileError: function (event) {

       var source = event.source === 'io' ? this.get('strings.error_network') : this.get('strings.error_http'),
           msg,
           test500 = /5\d+/,
           test400 = /4\d+/;

        switch(event.status) {

            case test500.test(event.status):
                msg = Y.Lang.sub(this.get('strings.error_server'), {
                    status: event.status
                });
                break;
            case test400.test(event.status):
                msg = Y.Lang.sub(this.get('strings.error_server'), {
                    status:         event.status,
                    status_text:    event.statusText
                });
                break;
            case 0:
                msg = this.get('strings.error_aborted');
                break;
            default:
                msg = Y.Lang.sub(this.get('strings.error_generic'), {
                    status:         event.status,
                    status_text:    event.statusText,
                    source:         source
                });
                break;
        }

        return {
            source: source,
            message: msg
        };

    },

    /**
    * Return whether or not a list containes the provided state
    * @param state {String} state to check
    * @param list {Array} list of files, defaults to this instance's fileList
    * @method containsState
    */
    containsState: function (state, list) {

        return !!Y.Array.find(list || this.get('fileList'), function (file) {
            return file.get('status') === state;
        });

    },

    /**
    * Return whether or not a list containes all given states
    * @param states {Array} list of states to check
    * @param list {Array} list of files, defaults to this instance's fileList
    * @method containsAllStates
    */
    containsAllStates: function (states, list) {
        var containsStates = true;

        Y.each(states, function (state) {
            if(!this.containsState(state, list)) {
                containsStates = false;
            }
        }, this);

        return containsStates;
    },

    /**
    * Return whether or not a list containes any of the given states
    * @param states {Array} list of states to check
    * @param list {Array} list of files, defaults to this instance's fileList
    * @method containsAnyStates
    */
    containsAnyStates: function (states, list) {

        return !!Y.Array.find(states, function (state) {
            return this.containsState(state, list);
        }, this);

    },

    containsOnlyState: function (state, list) {

        return Y.Array.every(list || this.get('fileList'), function (file) {
            return file.get('status') === state;
        });

    },

    /**
    * Return files in given state
    * @param state {String} state we look for
    * @param list {Array} list of files, defaults to this instance's fileList
    * @method getFilesInState
    */
    getFilesInState: function (state, list) {

        return Y.Array.filter(list || this.get('fileList'), function (file) {
            return file.get('status') === state;
        });

    },

    /**
    * Return files in given states
    * @param states {Array} states we look for
    * @param list {Array} list of files, defaults to this instance's fileList
    * @method getFilesInStates
    */
    getFilesInStates: function (states, list) {

        var files = [];

        Y.each(states, function (state) {
            files = files.concat(this.getFilesInState(state, list));
        }, this);

        return files;

    },

    testFileState: function (file, states) {
        if(!Y.Lang.isArray(states)) {
            states = [states];
        }
        return Y.Array.some(states, function (state) {
            return state === file.get('status');
        });
    },

    // files that exists only locally, has not yet been uploaded
    getLocalFiles: function (list) {

        return this.getFilesInStates([
            Y.U1.Widgets.Uploader.PENDING,
            Y.U1.Widgets.Uploader.FAILED,
            Y.U1.Widgets.Uploader.CANCELLED,
            Y.U1.Widgets.Uploader.STARTED
        ], list);

    },

    haveLocalFiles: function (list) {


        return this.containsAnyStates([
            Y.U1.Widgets.Uploader.PENDING,
            Y.U1.Widgets.Uploader.FAILED,
            Y.U1.Widgets.Uploader.CANCELLED,
            Y.U1.Widgets.Uploader.STARTED
        ], list);

    },

    getServerFiles: function (list) {

        return this.getFilesInState(Y.U1.Widgets.Uploader.COMPLETE, list);

    },

    haveServerFiles: function (list) {

        return this.containsState(Y.U1.Widgets.Uploader.COMPLETE, list);

    },

    getProgressingFiles: function (list) {

        return this.getFilesInStates([
            Y.U1.Widgets.Uploader.UPLOADING,
            Y.U1.Widgets.Uploader.STARTED
        ], list);

    },

    haveProgressingFiles: function (list) {

        return this.containsAnyStates([
            Y.U1.Widgets.Uploader.UPLOADING,
            Y.U1.Widgets.Uploader.STARTED
        ], list);

    },

    getFailedFiles: function (list) {

        return this.getFilesInState(Y.U1.Widgets.Uploader.FAILED, list);

    },

    haveFailedFiles: function (list) {

        return this.containsState(Y.U1.Widgets.Uploader.FAILED, list);

    },

    getCancelledFiles: function (list) {

        return this.getFilesInState(Y.U1.Widgets.Uploader.CANCELLED, list);

    },

    haveCancelledFiles: function (list) {

        return this.containsState(Y.U1.Widgets.Uploader.CANCELLED, list);

    },

    getPendingFiles: function (list) {

        return this.getFilesInState(Y.U1.Widgets.Uploader.PENDING, list);

    },

    havePendingFiles: function (list) {

        return this.containsState(Y.U1.Widgets.Uploader.PENDING, list);

    },

    getFilesTotals: function (list) {

        var filesCount      = 0,
            filesTotalSize  = 0,
            fileList        = list || this.get('fileList');

        Y.each(fileList, function (file) {
            filesCount++;
            filesTotalSize += file.get('size');
        });

        return {
            count: filesCount,
            size: filesTotalSize
        };

    },

    /**
    * Convert number of bytes into human readable string (ala "2 MB")
    * @method getFriendlySize
    * @param size the size in bytes
    *
    * https://github.com/sspencer/yui3-gallery
    * @param size {Number}
    * @method getFriendlySize
    */
    getFriendlySize: function(size) {

        var i, units = [
            this.get("strings.size_b"),
            this.get("strings.size_kb"),
            this.get("strings.size_mb"),
            this.get("strings.size_gb"),
            this.get("strings.size_tb")
        ];

        if(!size) {
            return 0;
        }

        for (i = 0; size > 1024; i++) { size /= 1024; }

        if (i < units.length) {
            return Math.round(size*10)/10 + units[i];
        } else {
            return "?";
        }
    }

}, {

    /**
    * Static constant for the value of the file `status` attribute:
    * The file has been added to the uploader
    * @property PENDING
    * @readOnly
    * @type {String}
    * @static
    */
    PENDING: 'PENDING',

    /**
    * Static constant for the value of the file `status` attribute:
    * The upload is currently being uploaded
    * @property UPLOADING
    * @readOnly
    * @type {String}
    * @static
    */
    UPLOADING: 'UPLOADING',

    /**
    * Static constant for the value of the file `status` attribute:
    * The file's upload has been started, i.e. queued, but not currently being uploaded
    * @property STARTED
    * @readOnly
    * @type {String}
    * @static
    */
    STARTED: 'STARTED',

    /**
    * Static constant for the value of the file `status` attribute:
    * The file's upload has been completed successfully
    * @property COMPLETE
    * @readOnly
    * @type {String}
    * @static
    */
    COMPLETE: 'COMPLETE',

    /**
    * Static constant for the value of the file `status` attribute:
    * The file's upload failed
    * @property FAILED
    * @readOnly
    * @type {String}
    * @static
    */
    FAILED: 'FAILED',

    /**
    * Static constant for the value of the file `status` attribute:
    * The file's upload was cancelled
    * @property CANCELLED
    * @readOnly
    * @type {String}
    * @static
    */
    CANCELLED: 'CANCELLED',

    ATTRS: {


        dragAndDropArea: { value: '.uploader-dd-proxy' },

        errorAction: { value: Y.Uploader.Queue.RESTART_ASAP },

        strings: {

            value: {



                cancel:             'Cancel',
                cancel_pending:     'Discard {count} pending files?',
                cancel_remaining:   'Stop uploading and discard {count} remaining files?',
                cancelled:          'Cancelled',
                complete:           'Complete',

                done:               'Done',
                drop_pending:       'Files detected, drop them here!',
                drop_ready:         'Drag and drop files here',

                error:              '{error} error',
                error_http:         'Server',
                error_network:      'Network',

                error_aborted:      'Upload request aborted.',
                error_client:       'Upload failed due to a client error {status}: {status_text}.',
                error_generic:      'Upload failed due to an {source} error. Details: {status_text} code {status}.',
                error_server:       'Upload failed due to a server error {status}.',

                failed:             'Failed',

                incomplete:         'No files were uploaded due to an error.',

                not_started:        'Not started',

                partialcomplete:    'Some files were not uploaded',

                size_b:             ' B',
                size_kb:            ' KiB',
                size_mb:            ' MiB',
                size_gb:            ' GiB',
                size_tb:            ' TiB',

                time_s:             'second',
                time_m:             'minute',
                time_h:             'hour',
                time_d:             'day',
                time_u:             'unknown time',
                time_plural:        's',
                time_remaining:     '<span>{time} remaining</span>',
                totaluploadprogress:'Total uploaded: <strong class="uploader-percentage">{percent}%</strong>',

                unknownprogress:    'Uploading',
                upload:             'Upload',
                upload_complete:    'Upload complete',
                fullycomplete:      'Uploads complete!',
                uploads_empty:      'No files selected for upload.',
                uploads_nofiles:    'There were no files to upload. Cancel and try again?',
                uploads_pending:    'Will upload <strong>{count}</strong> files, <strong>{size}</strong> in total.'


            }
        },

        /**
        * Uploader status presented as a string
        * This attribute tries to inform consumers
        * about what is currently happening
        *
        * @attribute status
        * @type {Object}
        */
        status: {
            value: {
                text: '',
                size: 0,
                count: 0,
                percent: null,
                state: undefined
            },
            getter: function (val) {

                val.text = Y.Lang.sub(val.text, {
                    count: val.count,
                    size: this.getFriendlySize(val.size),
                    percent: val.percent ? Math.floor(val.percent) : val.percent
                });

                return val;
            }
        },

        /**
        * Files drop area status as a string
        *
        * @attribute dropAreaStatus
        * @type {String}
        * @default
        */
        dropAreaStatus: { value: '' }

    }


});

Y.namespace('U1.Widgets').Uploader = Uploader;


}, '@VERSION@', {"requires": ["array-extras", "uploader-html5"]});
