'use strict';

(function() {
    describe('ga-tracking', function() {
        var Y;
       var t;

        before(function(done) {
            Y = YUI().use(
                'ga-tracking', 'node-event-simulate', function(Y) {
                Y.one('body').append('<div id="test"/>');
                t = Y.one('#test');
                done();
            });
        });

        afterEach(function (done) {

            Y.GaTracking.destroy();
            done();

        });


        beforeEach(function (done) {

            Y.config.win._gaq = [];

            t.empty();
            t.append(Y.Node.create('<div class="test-container"><a class="test-link" href="#test-link">Test link</a></div>'));
            done();

        });

        after(function (done) {

            t.remove(true);
            done();

        });

        describe('Y.GaTracking', function () {

            it('should be loaded', function() {
                expect(Y.GaTracking).to.exist;
            });

        });

        describe('#destroy()', function () {

            it('should remove configs', function() {

                Y.GaTracking.add({
                    action: 'Test Click',
                });

                Y.GaTracking.track();
                Y.GaTracking.destroy();

                expect(Y.GaTracking.configs.length).to.eql(0);

            });

            it('should remove listeners', function() {

                Y.GaTracking.add({
                    action: 'Test Click',
                });

                Y.GaTracking.track();
                Y.GaTracking.destroy();

                Y.one('#test a').simulate('click');

                expect(Y.config.win._gaq.length).to.eql(0);

            });

        });

        describe('#collect()', function () {

            var target;
            var targetGaLabel;

            beforeEach(function (done) {

                target = Y.Node.create('<a href="bar">Bar</a>');
                targetGaLabel = Y.Node.create('<a href="foo" data-ga-label="Foo"></a>');

                done();

            });

            it('should return label', function() {

                var collected = Y.GaTracking.defaults.collect(target);

                expect(collected.link).to.eql('bar');
                expect(collected.label).to.eql('Bar');

            });

            it('should return data-ga-label if existant', function() {

                var collected = Y.GaTracking.defaults.collect(targetGaLabel);

                expect(collected.link).to.eql('foo');
                expect(collected.label).to.eql('Foo');

            });

        });

        describe('#push()', function () {

            it('should add to queue', function () {

                Y.GaTracking.defaults.push('foo', 'bar');
                expect(Y.config.win._gaq[0]).to.include.members(['_trackEvent', 'foo', 'bar']);

            });

        });

        describe('#add()', function () {

             it('should be able to add track configs', function (){

                Y.GaTracking.add({
                    action: 'Test Click',
                    target: '.test-link',
                    container: '.test-container'
                });

            });

            it('should push to queue on click', function (){

                Y.GaTracking.add({
                    action: 'Test Click',
                    target: '.test-link',
                    container: '.test-container'
                });

                Y.GaTracking.track();

                Y.one('#test a').simulate('click');

                expect(Y.config.win._gaq[0]).to.include.members(['Test Click']);
                expect(Y.config.win._gaq.length).to.eql(1);

            });

            it('should not die given bad container', function (){

                Y.GaTracking.add({
                    container: '.bad-container'
                });

                Y.GaTracking.track();

            });

            it('should be able to override tracking functionality', function (){

                var pushCalled = false;
                var collectCalled = false;

                Y.GaTracking.add({
                    action: 'Test Click',
                    target: '.test-link',
                    container: '.test-container',
                    push: function () {
                        pushCalled = true;
                    },
                    collect: function () {
                        collectCalled = true;
                        return {link: '#foo', label: 'bar'};
                    }
                });

                Y.GaTracking.track();

                Y.one('#test a').simulate('click');

                expect(pushCalled).to.be.true;
                expect(collectCalled).to.be.true;

            });

            it('should be able to manually override label', function (){

                Y.GaTracking.add({
                    action: 'Test Click',
                    target: '.test-link',
                    container: '.test-container',
                    label: 'foobar'
                });

                Y.GaTracking.track();

                Y.one('#test a').simulate('click');

                expect(Y.config.win._gaq[0]).to.include.members(['foobar']);

            });

            it('should be add multiple', function (){

                Y.one('.test-container').append('<a class="extra-link" href="#extra">Extra link</a>');

                Y.GaTracking.add({
                    category: 'A',
                    action: 'Test Click',
                    target: '.test-link',
                    container: '.test-container'
                });

                Y.GaTracking.add({
                    category: 'B',
                    action: 'Test Click',
                    target: '.extra-link',
                    container: '.test-container'
                });

                Y.GaTracking.track();

                Y.one('#test .test-link').simulate('click');
                Y.one('#test .extra-link').simulate('click');

                expect(Y.config.win._gaq[0]).to.include.members(['A', 'Test link']);
                expect(Y.config.win._gaq[1]).to.include.members(['B', 'Extra link']);

            });

        });

    });
})();
