/**
 * A password strength meter widget.
 *
 *
 * Given the id of a password field and the name of a function that will check
 * the strength of the input password this meter will display on each character
 * entered the strength value computed by the function provided.
 *
 * @module passwordmeter
 */

 // TODO: slidedown/up should be plugged in
 var slideDown = {
        height: function(node) {
            return node.get('scrollHeight') + 'px';
        },
        duration: 0.1,
        easing: 'ease-out',
        on: {
            start: function() {
                var overflow = this.getStyle('overflow');
                if (overflow !== 'hidden') { // enable scrollHeight/Width
                    this.setStyle('overflow', 'hidden');
                    this._transitionOverflow = overflow;
                }
            },
            end: function() {
                if (this._transitionOverflow) { // revert overridden value
                    this.setStyle('overflow', this._transitionOverflow);
                    delete this._transitionOverflow;
                }
            }
        }
    },
    slideUp = {
        height: 0,
        duration: 0.1,
        easing: 'ease-out',
        on: {
            start: function () {
                this.setStyle('overflow', 'hidden');
            }
        }
    },

    ASYMMETRIC = 'asymmetric',
    SHORT = 'short',
    LONG_PASSWORD = 12,

    PasswordMeter,

    /**
     * This function will be used to determine the password's strength. It only
     * gives a 100% strength if the password has a combination of lower case, upper
     * case, numeric and symbolic characters with at least a length of 8.
     * The function can be replaed by the user of the widget by simply supplying
     * their own function in the "func" attribute.
     * The function returns two values: strength which is an int between 0 and 100
     * and text which is one of the following strings: weak, medium, strong.
     */
     default_password_strength_function = function(password) {
        var hasLower, hasUpper, hasLowerAndUpper, hasNumber, hasSymbol, hasLength, sum;

        hasLower = /[a-z]/.test(password);
        hasUpper = /[A-Z]/.test(password);
        hasLowerAndUpper = hasLower && hasUpper ? 1 : 0;
        hasNumber = /\d/.test(password) ? 1 : 0;

        // add to sum if password includes one of these symbols
        hasSymbol = /[\~\`\!\@\#\$\%\^\&\*\(\)\_\-\+\=\{\}\[\]\:\;\"\'<\>\?\|\\\,\.\/]/.test(password) ? 1 : 0;

        // add to sum if password is substantially long
        hasLength = password.length >= LONG_PASSWORD ? 1 : 0;

        sum = hasLowerAndUpper + hasNumber + hasSymbol + hasLength;

        return sum;
    };


    PasswordMeter = Y.Base.create('u1PasswordMeter', Y.Widget, [], {

        _indicatorNode: null,

        initializer: function () {

            this.on('strengthChange', this._strengthChange, this);

            // revert input position
            this.before('destroy', function () {
                var input = this.get('input'),
                bb = this.get('boundingBox');

                bb.insert(input, 'before');
            });

        },



    /**
     * Handler for change events from the strength attribute
     *
     * Checks the new strength value and re-renders the indicatorNode
     *
     * @method renderUI
     * @protected
     */
     _strengthChange: function (e) {

        // no change
        if(e.prevVal === e.newVal) {
            return;
        }

        var feedback,
            strings = this.get('feedback'),
            strengths = strings.strengths,
            bb = this.get('boundingBox');

        if(Y.Lang.isNumber(e.newVal)) {
            feedback = strengths[e.newVal >= strengths.length ? strengths.length - 1 : e.newVal];
        } else {
            feedback = this.get('feedback')[e.newVal];
        }

        if(!feedback) {
            return;
        }

        // nuke strength className on indicator
        // add new className
        this._removeStrengthClass(bb);
        bb.addClass(this.getClassName('strength', feedback.className));

        // nuke content, add new text
        this._indicatorNode.one('p').setContent(feedback.text);

    },

    // TODO: improve, replace with classmanager
    _removeStrengthClass: function (n) {
        var a = n.getAttribute('class');
        n.setAttribute('class', a.replace(/\w+-\w+-strength-\w+/, ''));
        // clean whitespace
        n.setAttribute('class', n.getAttribute('class').replace(/\s{2,}/, ''));
    },

    /**
     * Update the DOM structure and edit CSS classes.
     *
     * @method renderUI
     * @protected
     */
     renderUI: function(){

        this._renderInput();

        this._renderIndicator();

    },

    _renderInput: function () {

        var input = this.get('input'),
        bb = this.get('boundingBox'),
        cb = this.get('contentBox');

        // move input to contentBox
        input.insert(bb, input);
        cb.append(input);

        return input;

    },

    _renderIndicator: function () {

        var indicatorNode = Y.Node.create('<div></div>');

        // add class name to indicator
        indicatorNode.addClass(this.getClassName('indicatorNode'));

        // insert indicator
        this.get('input').insert(indicatorNode, 'after');

        // use width attribute
        indicatorNode.setStyle('marginLeft', this.get('input').getComputedStyle('marginLeft'));
        indicatorNode.setStyle('marginRight', this.get('input').getComputedStyle('marginRight'));
        indicatorNode.setStyle('paddingLeft', this.get('input').getComputedStyle('paddingLeft'));
        indicatorNode.setStyle('paddingRight', this.get('input').getComputedStyle('paddingRight'));

        // add meter and text
        indicatorNode.setContent('<div></div><p data-qa-id="password_strength"></p>');

        // make property
        this._indicatorNode = indicatorNode;

        return indicatorNode;

    },

    /**
     * Set the event handlers for the input element.
     *
     * @method bindUI
     * @protected
     */
     bindUI: function(){

        this.get('input').on(['keyup', 'focus', 'blur'], this._inputEvent, this);

        if(this.get('inputConfirm')) {
            this.get('inputConfirm').after(['blur', 'change'], this._inputConfirmEvent, this);
        }

    },

    /**
     * Synchronize the DOM with our current attribute state
     *
     * @method syncUI
     * @protected
     */
     syncUI: function() {


    },


    /**
     * Handler for an event in the main input
     *
     * @method _inputEvent
     * @protected
     */
    _inputEvent: function () {

        this._setStrength();

        this._toggleIndicatorVisibility();

    },


    /**
     * Transitions the indicator based on input length and state
     *
     * @method _toggleIndicatorVisibility
     * @param force {Boolean} force visible
     * @protected
     */
    _toggleIndicatorVisibility: function (force) {

        // show meter after n chars
        // only transition if needed
        if(this.get('input').get('value').length > this.get('checkThreshold') || force) {
            if(!this.get('visibility')) {
                this._indicatorNode.transition(slideDown);
                this.set('visibility', true);
            }
        } else {
            if(this.get('visibility')) {
                this._indicatorNode.transition(slideUp);
                this.set('visibility', false);
            }
        }


    },

    /**
     * Re-sets the strength attribute with result from
     * passing the input value to our passwordChecker function
     *
     * @method _setStrength
     * @protected
     */
     _setStrength: function () {
        var pwd = this.get('input').get('value');

        if(pwd && pwd.length >= this.get('minimumLength')) {
            this.set('strength', this.get('passwordChecker')(pwd));
        } else {
            this.set('strength', SHORT);
        }

    },


    /**
     * Handler for an event in the confirmation input
     *
     * @method _inputConfirmEvent
     * @protected
     */
    _inputConfirmEvent: function () {
        var one = this.get('input').get('value'),
            two = this.get('inputConfirm').get('value');

        // if equal
        if(one === two) {
            // trigger event on main input
            this._inputEvent();
        } else {
            this.set('strength', ASYMMETRIC);
            this._toggleIndicatorVisibility(true);
        }
    }

}, {
    ATTRS: {

        /**
         * Password field we will be monitoring
         *
         * @attribute input
         * @type Node
         * @default null
         */
         input: {
            value: null,
            setter: function(v){
                return Y.one(v);
            }
        },


        /**
         * A input that will have to match the
         * main input
         *
         * @attribute inputConfirm
         * @type Node
         * @default null
         */
         inputConfirm: {
            value: null,
            setter: function(v){
                return Y.one(v);
            }
        },

        /**
         * Current password strength
         *
         * @attribute strength
         * @type Number
         * @default null
         */
         strength: {
            value: null
        },

        /**
         * Minimum password length
         *
         * @attribute minimumLength
         * @type Number
         * @default 8
         */
         minimumLength: {
            value: 8,
            writeOnce: 'initOnly',
            validator: function (val) {
                return Y.Lang.isNumber(val);
            }
        },

        /**
         * When to start checking the password
         *
         * @attribute checkThreshold
         * @type Number
         * @default 0
         */
         checkThreshold: {
            value: 0,
            writeOnce: 'initOnly',
            validator: function (val) {
                return Y.Lang.isNumber(val);
            }
        },

        /**
         * Javascript function that will calculate the password's strength
         * and strength description
         * Should return an object literal in the form
         * {"color": <color of the text displaying the strength>,
         *  "text": <textual description of the strength>}
         *
         * @attribute passwordChecker
         * @type function
         * @default a standard password strength function
         */
         passwordChecker: {
            value: default_password_strength_function,
            validator: function(val) {
                return Y.Lang.isFunction(val);
            }
        },

        /**
         * Array defining how the various strengths are to be
         * communicated to the user
         *
         * TODO: use strings pattern instead
         *
         * @attribute passwordChecker
         * @type function
         * @default a standard password strength function
         */
         feedback: {
            value: {
                strengths: [
                    {
                        className: 'weak',
                        text: 'Strength: weak'
                    },
                    {
                        className: 'fair',
                        text: 'Strength: fair'
                    },
                    {
                        className: 'good',
                        text: 'Strength: good'
                    },
                    {
                        className: 'strong',
                        text: 'Strength: strong'
                    }
                ],
                'short': {
                    className: 'short',
                    text: 'Password is too short'
                },
                'asymmetric': {
                    className: 'asymmetric',
                    text: 'The passwords do not match'
                }
            }
        }

    }
});

/**
 * The HTML_PARSER static constant is used by the Widget base class to
 * populate the configuration for the PasswordMeter instance from markup
 * already on the page.
 *
 * @property PasswordMeter.HTML_PARSER
 * @type Object
 * @static
 */
 PasswordMeter.HTML_PARSER = {};

 PasswordMeter.NAME = 'passwordmeter';

 Y.namespace('U1.Widgets').PasswordMeter = PasswordMeter;
