'use strict';

(function() {
    describe('uploader-myapps-packages', function() {
        var Y;
        var uploader;
        var html;

        before(function(done) {

            Y = YUI().use(
                'uploader-myapps-packages', 'file-html5', function(Y) {
                    html = Y.Node.create('<div id="test" class="devportal-form" />');
                    Y.one('body').append(html);
                    done();
            });

        });

        beforeEach(function (done) {

            uploader = new Y.UploaderMyappsPackages();

            // create required structure
            html.append(Y.Node.create('<div class="field field-binary"><div class="field-content"/></div>'));
            html.append(Y.Node.create('<div class="field field-source"><div class="field-content"/></div>'));
            html.append(Y.Node.create('<button type="submit" />'));

            uploader.render();

            done();

        });

        afterEach(function (done) {

            uploader.destroy();
            html.empty();
            done();

        });

        describe('#initializer()', function () {

            it('should be loaded', function() {

                expect(Y.UploaderMyappsPackages).to.exist;

            });

            it('should be instansiable', function() {

                expect(uploader).to.exist;

            });

            it('should create a binary uploader', function() {

                expect(uploader._binaryUploader).to.exist;

            });

            it('should create a source uploader', function() {

                expect(uploader._sourceUploader).to.exist;

            });

        });

        describe('#submitHandler()', function () {

            var mockE,
                updateVersionProto;

            beforeEach(function (done) {

                mockE = {
                    preventDefault: function () {}
                };

                if(!updateVersionProto) {
                    updateVersionProto = Y.UploaderMyappsPackages.prototype.updateVersion;
                }

                Y.UploaderMyappsPackages.prototype.updateVersion = function () {
                    return true;
                }

                done();

            });

            after(function (done) {

                Y.UploaderMyappsPackages.prototype.updateVersion = updateVersionProto;
                done();

            });

            it('should fire clear event', function() {

                var called = false;

                uploader.on('clear', function () {
                    called = true;
                });

                uploader.submitHandler(mockE);
                expect(called).to.be.true;

            });

            it('should fire binaryisnullbyte when binary has no size', function () {

                var called = false;

                uploader.on('binaryisnullbyte', function () {
                    called = true;
                });

                uploader._binaryUploader.set('fileList', [ new Y.FileHTML5({ name: 'foo', size: 0 })]);

                uploader.submitHandler(mockE);
                expect(called).to.be.true;

            });

            it('should fire nobinary when there is no binary file', function () {

                var called = false;

                uploader.on('nobinary', function () {
                    called = true;
                });

                uploader.submitHandler(mockE);
                expect(called).to.be.true;

            });

        });

        describe('#updateVersion()', function () {

            var mockXHR;
            var responseJSON;

            beforeEach(function (done) {

                uploader.set('strings', {
                    could_not_process: 'could_not_process'
                });


                Y.each(['id_binary_path', 'id_source_path', 'id_signature'], function (item) {
                    html.append(Y.Node.create('<input id="' + item + '" />'));
                });

                // do some manual mocking

                mockXHR = {
                    send: function () {
                        return {
                            responseText: responseJSON
                        };
                    },
                };

                Y.IO.transports.mock = function () {
                    return mockXHR;
                }

                Y.UploaderMyappsPackages.prototype.getCallbackConfig = function () {
                    return {
                        xdr: {
                            use: 'mock'
                        },
                        on: {
                            failure : Y.bind(function () {

                                this.fire('error', {
                                    message: 'could_not_process'
                                });

                            }, uploader)
                        }
                    };
                };

                done();

            });

            it('should fire error if failing to parse payload', function () {

                var error;

                responseJSON = undefined;

                uploader.on('error', function (e) {
                    error = e;
                })

                uploader.updateVersion();

                expect(error.message).to.eql('could_not_process');

            });


            it('should fire error response not success', function () {

                var error;

                responseJSON = Y.JSON.stringify({
                    errors: {
                        'binary': 'fail'
                    }
                });

                uploader.on('error', function (e) {
                    error = e;
                })

                uploader.updateVersion();

                expect(error.message).to.eql('fail');
                expect(error.field).to.eql('.field-binary');

            });

            it('should update DOM elements on success', function () {

                responseJSON = Y.JSON.stringify({
                    success: true,
                    binary_path: 'bin',
                    source_path: 'src',
                    signature: 'sign'
                });

                expect(uploader.updateVersion()).to.be.true;

                expect(Y.one('#id_binary_path').get('value')).to.eql('bin');
                expect(Y.one('#id_source_path').get('value')).to.eql('src');
                expect(Y.one('#id_signature').get('value')).to.eql('sign');
            });

        });

        describe('Errors', function () {

            beforeEach(function (done) {

                uploader.set('strings', {
                    binary_none: 'binary_none',
                    binary_nullbyte: 'binary_nullbyte',
                    already_uploaded: 'already_uploaded',
                    correct_errors: 'correct_errors',
                    could_not_process: 'could_not_process'
                });

                done();

            });

            describe('Binary', function () {

                it('should emit generic error on nobinary', function () {

                    var error;

                    uploader.once('error', function (e) {
                        error = e;
                    });

                    uploader.fire('uploaderMyappsPackage:nobinary');

                    expect(error.field).to.be.undefined;
                    expect(error.message).to.eql('correct_errors');

                });

                it('should emit field specific error on nobinary', function () {

                    var error;

                    // catch both generic and specific, but set error to latter
                    uploader.on('error', function (e) {
                        error = e;
                    });

                    uploader.fire('uploaderMyappsPackage:nobinary');

                    expect(error.field).to.eql('.field-binary');
                    expect(error.message).to.eql('binary_none');

                });

                it('should emit generic error on binaryisnullbyte', function () {

                    var error;

                    uploader.once('error', function (e) {
                        error = e;
                    });

                    uploader.fire('uploaderMyappsPackage:binaryisnullbyte');

                    expect(error.field).to.be.undefined;
                    expect(error.message).to.eql('correct_errors');

                });

                it('should emit field specific error on binaryisnullbyte', function () {

                    var error;

                    // catch both generic and specific, but set error to latter
                    uploader.on('error', function (e) {
                        error = e;
                    });

                    uploader.fire('uploaderMyappsPackage:binaryisnullbyte');

                    expect(error.field).to.eql('.field-binary');
                    expect(error.message).to.eql('binary_nullbyte');

                });

                it('should emit generic error on binaryisnotunique', function () {

                    var error;

                    uploader.once('error', function (e) {
                        error = e;
                    });

                    uploader.fire('uploaderMyappsPackage:binaryisnotunique');

                    expect(error.field).to.be.undefined;
                    expect(error.message).to.eql('correct_errors');

                });

                it('should emit field specific error on binaryisnotunique', function () {

                    var error;

                    // catch both generic and specific, but set error to latter
                    uploader.on('error', function (e) {
                        error = e;
                    });

                    uploader.fire('uploaderMyappsPackage:binaryisnotunique');

                    expect(error.field).to.eql('.field-binary');
                    expect(error.message).to.eql('already_uploaded');

                });

                it('should re-create sourceUploader on binaryisnotunique', function () {

                    uploader._sourceUploader.destroy();
                    uploader._sourceUploader = null;


                    uploader.fire('uploaderMyappsPackage:binaryisnotunique');

                    expect(uploader._sourceUploader).to.not.be.null;

                });

            });


            describe('Upload', function () {

                var mockE;

                beforeEach(function (done) {

                    mockE = {
                        context: uploader._binaryUploader,
                        originEvent: {
                            source: 'io',
                            status: '500',
                            statusText: 'Failed horribly'
                        }
                    };

                    uploader._binaryUploader.set('strings.error_generic', 'error_generic {status} {status_text}');

                    done();

                });

                it('should fire generic error on upload failure', function () {

                    var file = new Y.FileHTML5({
                        name: 'foo'
                    });
                    var error;

                    uploader.once('error', function (e) {
                        error = e;
                    });

                    uploader.fire('myappsUploader:uploadfailed', mockE);

                    expect(error.message).to.eql('correct_errors');
                    expect(error.field).to.be.undefined;

                });

                it('should fire field specific error on upload failure', function () {

                    var file = new Y.FileHTML5({
                        name: 'foo'
                    });
                    var error;

                    // catch both generic and specific, but set error to the latter
                    uploader.on('error', function (e) {
                        error = e;
                    });

                    uploader.fire('myappsUploader:uploadfailed', mockE);

                    expect(error.message).to.eql('error_generic 500 Failed horribly');
                    expect(error.field).to.not.be.null;

                });

            });

        });
    });

})();
