var ExpandableLabel,
    SelectRelation,
    CheckboxDependency,
    DPErrors;

/**
 * A class for expandable labels
 *
 * @class ExpandableLabel
 * @extends Object
 * @namespace Devportal.Utils
 */
ExpandableLabel = function (labelId, expandedInitially) {

    this.label = this.getLabel(labelId);

    this.labelRow = this.getLabelRow(this.label);

    // TODO: static
    this.classes = {
        expanded: 'expandablelabel-expanded',
        collapsed: 'expandablelabel-collapsed',
        base: 'expandablelabel'
    };

    this.expandedInitially = Y.Lang.isBoolean(expandedInitially) ? expandedInitially : false;

    this.render();
    this.bind();

};

ExpandableLabel.prototype.getLabel = function (id) {
    return Y.one('label[for='+id+']');
};

ExpandableLabel.prototype.getLabelRow = function (label) {
    return label.ancestor('.field');
};

ExpandableLabel.prototype.collapse = function () {

    var c = this.classes;
    this.labelRow
        .addClass(c.collapsed)
        .removeClass(c.expanded);

};

ExpandableLabel.prototype.expand = function () {

    var c = this.classes;
    this.labelRow
        .removeClass(c.collapsed)
        .addClass(c.expanded);

};

ExpandableLabel.prototype.render = function () {

    // add base class
    this.labelRow.addClass(this.classes.base);

    // expand / collapse class
    if(this.expandedInitially) {
        this.expand();
    } else {
        this.collapse();
    }

};

ExpandableLabel.prototype.bind = function () {

    this.labelRow.delegate('click', this.click, '.'+this.classes.base, this);

};

ExpandableLabel.prototype.click = function (event) {

    var ct = event.currentTarget,
        cls = this.classes;

    // if expanded, only collapse if the target is a label and direct descendant of expandablelabel
    if(ct.hasClass(cls.expanded) && event.target.test('.'+cls.base+' .field-label')) {

        event.halt();
        this.collapse();

    } else if(ct.hasClass(cls.collapsed)) {

        event.halt();
        this.expand();

    }


};

ExpandableLabel.prototype.collapsedClick = function () {

    this.expand();

};

ExpandableLabel.prototype.expandedClick = function () {

    this.collapse();

};

/**
 * A class that lets two select boxes exclude each other
 *
 * @class SelectRelation
 * @extends Object
 * @namespace Devportal.Utils
 */
SelectRelation = function (select, antiSelect) {
    this.select = select;
    this.antiSelect = antiSelect;

    this.sync();
    this.bind();
};

SelectRelation.prototype.sync = function () {

    this.antiSelect.set('disabled', this.select.get('selectedIndex') > -1);

};

SelectRelation.prototype.bind = function () {

    this.select.on('change', this.change, this);

};

SelectRelation.prototype.change = function () {

    this.sync();

};

/**
 * A class that defines a checkbox dependant on another
 *
 * @class CheckboxDependency
 * @extends Object
 * @namespace Devportal.Utils
 */
CheckboxDependency = function (depender, dependee) {

    this.depender = depender;
    this.dependee = dependee;

    this.sync();
    this.bind();
};

CheckboxDependency.prototype.sync = function () {

    this.dependee.set('disabled', !this.depender.get('checked'));

};

CheckboxDependency.prototype.bind = function () {

    this.depender.on('change', this.change, this);

};

CheckboxDependency.prototype.change = function () {

    this.dependee.set('checked', false);
    this.sync();
};

/**
 * A class for handling errors in a devportal form
 *
 * @class Errors
 * @extends Y.Base
 * @namespace Devportal.Utils
 */
DPErrors = Y.Base.create('devportalUtilsErrors', Y.Base, [], {

    initializer: function (cfg) {

        if(!cfg || !cfg.notifiers || cfg.notifiers.length === 0) {
            throw new Error('No notifiers given.');
        }

        Y.each(cfg.notifiers, function (notifier) {
            notifier.addTarget(this);
        }, this);

        this.on('*:error', this.create);
        this.on('*:clear', this.clear);

    },

    create: function (cfg) {

        var errors = cfg.message,
            field = cfg.field;

        if(!Y.Lang.isArray(errors)) {
            errors = [errors];
        }

        if(field) {
            field = Y.Lang.isString(field) ? this.get('form').one(field) : field;
        }

        this._create(errors, field);

    },

    // create form or field errors
    _create: function (errors, field) {

        var reciever,
            tmpl = this.get('template');

        if(field) {
            this.markField(field, true);
            reciever = field.one(this.get('fieldErrorListCss') + ' .errorlist');

            if(!reciever) {
                reciever = Y.Node.create(this.get('errorListTemplate'));
                field.one(this.get('fieldErrorListCss')).append(reciever);
            }

        } else {
            reciever = this.get('formErrors');
        }

        Y.each(errors, function (error) {

            var node = Y.Node.create(Y.Lang.sub(tmpl, {
                message: error,
                'class': field ? 'field-error' : 'form-error'
            }));

            reciever.append(node);

        }, this);

    },

    // clears field error, or all errors
    clear: function (cfg) {

        var field = cfg.field;

        if(field) {
            Y.one(field).one(this.get('fieldErrorListCss')).empty();
            this.markField(field, false);
        } else {
            Y.all(this.get('fieldErrorListCss') + ', .form-errors').empty();
            this.markAllFields(false);
        }

    },

    // mark all fields
    markAllFields: function (hasErrors) {
        Y.all('.field').each(function (field) {
            this.markField(field, hasErrors);
        }, this);
    },

    // mark field either with an error, or without
    markField: function (field, hasErrors) {

        var cls = 'has-errors';

        if(hasErrors) {
            field.addClass(cls);
        } else {
            field.removeClass(cls);
        }
    }

}, {

    ATTRS: {
        errorListTemplate: { value: '<ul class="errorlist" />' },
        fieldErrorListCss: { value: '.field-errors' },
        form: {
            value: null,
            setter: function (val) {
                return Y.Lang.isString(val) ? Y.one(val) : val;
            }
        },
        formErrors: {
            value: null,
            setter: function (val) {
                return Y.Lang.isString(val) ? Y.one(val) : val;
            }
        },
        template: { value: '<li class="{class}">{message}</li>' }
    }

});


Y.namespace('Util').ExpandableLabel = ExpandableLabel;
Y.namespace('Util.Utils').SelectRelation = SelectRelation;
Y.namespace('Util').CheckBoxDependency = CheckboxDependency;
Y.namespace('Util').DPErrors = DPErrors;
